<?php
/**
 *  Copyright (C) Prestalia - All Rights Reserved
 *  Unauthorized copying of this file, via any medium is strictly prohibited
 *  Proprietary and confidential
 *
 *  @author    Prestalia <prestalia.it>
 *  @copyright 2019 - 2024 Prestalia
 *  @license   Commercial
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class SniperCommon
{
    public static function installTab($parent, $className, $name, $thisName)
    {
        // Create new admin tab
        $tab = new Tab();

        $tab->id_parent = (int) Tab::getIdFromClassName($parent);
        $tab->name = [];

        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = $name;
        }

        $tab->class_name = $className;
        $tab->module = $thisName;
        $tab->active = 1;

        return $tab->add();
    }

    public static function uninstallTab($className)
    {
        // Retrieve Tab ID
        $idTab = (int) Tab::getIdFromClassName($className);

        // Load tab
        $tab = new Tab((int) $idTab);

        // Delete it
        return $tab->delete();
    }

    public static function checkConfig($idShop, $idLang)
    {
        // 'key' => 'default value' pairs
        $config = [
            'enable_search' => '0',
            'sniper_default' => '1',
            'exports_gap' => '5',
            'available_filter' => '0',
            'available_filter_name' => 'Stock',
            'available_filter_instock' => 'In stock',
            'available_filter_outstock' => 'Out of stock',
        ];

        foreach ($config as $key => $val) {
            if (self::getConfig($key, $idShop, $idLang) == '') {
                self::setConfig($key, $val, $idShop, $idLang);
            }
        }

        if (self::getConfig('ajax_token', $idShop, $idLang) == '') {
            self::generateNewAjaxToken($idShop, $idLang, 'ajax_token');
        }

        if (self::getConfig('price_token', $idShop, $idLang) == '') {
            self::generateNewAjaxToken($idShop, $idLang, 'price_token');
        }

        if (self::getConfig('manager_token', $idShop, $idLang) == '') {
            self::generateNewAjaxToken($idShop, $idLang, 'manager_token');
        }

        self::setFiltersPosition($idShop, $idLang);

        $sql = 'SELECT type FROM ' . _DB_PREFIX_ . 'sniperfast_relevance_position
            WHERE id_shop = ' . (int) $idShop .
            ' AND id_lang = ' . (int) $idLang;
        $res = Db::getInstance()->executeS($sql);
        if (empty($res)) {
            self::initRelevancePosition($idShop, $idLang);
        }
    }

    public static function setFiltersPosition($idShop, $idLang)
    {
        $db = Db::getInstance();
        $dbp = _DB_PREFIX_;
        $sql = "INSERT IGNORE INTO {$dbp}sniperfast_filters_position (id_shop, id_lang, type, value, position)
         VALUES (" .
             (int) $idShop . ', ' .
             (int) $idLang . ', ' .
             "'pdata', " .
             '1, ' .
             '0' .
         '),
         (' .
             (int) $idShop . ', ' .
             (int) $idLang . ', ' .
             "'pdata', " .
             '2, ' .
             '1' .
         '),
         (' .
             (int) $idShop . ', ' .
             (int) $idLang . ', ' .
             "'pdata', " .
             '3, ' .
             '2' .
         '),
         (' .
             (int) $idShop . ', ' .
             (int) $idLang . ', ' .
             "'feat', " .
             '9950, ' .
             '3' .
         ')
         ';
        $db->execute($sql);

        $sql = "SELECT MAX(position)
            FROM {$dbp}sniperfast_filters_position
            WHERE id_shop = " . (int) $idShop .
           ' AND id_lang = ' . (int) $idLang;
        $maxPosition = $db->getValue($sql);

        $selectedFeatures = self::getConfig('sniper_features', $idShop, $idLang);
        if (empty($selectedFeatures)) {
            $sql = "UPDATE {$dbp}sniperfast_filters_position
                SET active = 0
                WHERE id_shop = " . (int) $idShop .
                ' AND id_lang = ' . (int) $idLang .
                " AND type = 'feat'";
            $db->execute($sql);
        } else {
            $selectedFeaturesExploded = explode(',', $selectedFeatures);
            $sql = "INSERT IGNORE INTO {$dbp}sniperfast_filters_position(id_shop, id_lang, type, value, position)
                VALUES ";
            $featureValues = [];
            foreach ($selectedFeaturesExploded as $sf) {
                $tmpVal = '(' .
                (int) $idShop . ', ' .
                    (int) $idLang . ', ' .
                    "'feat', " .
                    (int) $sf . ', ' .
                    (int) ++$maxPosition .
                ')';
                $featureValues[] = $tmpVal;
            }
            $db->execute($sql . implode(',', $featureValues));

            $sql = "UPDATE {$dbp}sniperfast_filters_position
                SET active = 1
                WHERE id_shop = " . (int) $idShop .
                ' AND id_lang = ' . (int) $idLang .
                " AND type = 'feat'
                  AND value IN (" . pSQL($selectedFeatures) . ')';
            $db->execute($sql);

            // Deactivate features no longer exported
            $sql = "UPDATE {$dbp}sniperfast_filters_position
                SET active = 0
                WHERE id_shop = " . (int) $idShop .
                ' AND id_lang = ' . (int) $idLang .
                " AND type = 'feat'
                  AND value NOT IN (" . pSQL($selectedFeatures) . ')';
            $db->execute($sql);

            // Deactivate features that no longer exist
            $sql = "UPDATE {$dbp}sniperfast_filters_position
                SET active = 0
                WHERE id_shop = " . (int) $idShop .
                ' AND id_lang = ' . (int) $idLang .
                " AND type = 'feat'
                  AND value NOT IN (
                      SELECT id_feature
                      FROM {$dbp}feature
                  )";
            if (in_array(9950, $selectedFeaturesExploded)) {
                $sql .= ' AND value != 9950';
            }
            $db->execute($sql);
        }

        $selectedAttributes = self::getConfig('sniper_attributes', $idShop, $idLang);
        if (empty($selectedAttributes)) {
            $sql = "UPDATE {$dbp}sniperfast_filters_position
                SET active = 0
                WHERE id_shop = " . (int) $idShop .
                ' AND id_lang = ' . (int) $idLang .
                " AND type = 'attr'";
            $db->execute($sql);
        } else {
            // Add new attributes to the filters order
            $selectedAttributesExploded = explode(',', $selectedAttributes);
            $sql = "INSERT IGNORE INTO {$dbp}sniperfast_filters_position (id_shop, id_lang, type, value, position)
                VALUES ";
            $attributeValues = [];
            foreach ($selectedAttributesExploded as $af) {
                $tmpAttr = '(' .
                    (int) $idShop . ', ' .
                    (int) $idLang . ', ' .
                    "'attr', " .
                    (int) $af . ', ' .
                    (int) ++$maxPosition .
                ')';
                $attributeValues[] = $tmpAttr;
            }
            $db->execute($sql . implode(',', $attributeValues));

            $sql = "UPDATE {$dbp}sniperfast_filters_position
                SET active = 1
                WHERE id_shop = " . (int) $idShop .
                ' AND id_lang = ' . (int) $idLang .
                " AND type = 'attr'
                AND value IN (" . pSQL($selectedAttributes) . ')';
            $db->execute($sql);

            // Deactivate attributes no longer exported
            $sql = "UPDATE {$dbp}sniperfast_filters_position
                SET active = 0
                WHERE id_shop = " . (int) $idShop .
                ' AND id_lang = ' . (int) $idLang .
                " AND type = 'attr'
                AND value NOT IN (" . pSQL($selectedAttributes) . ')';
            $db->execute($sql);

            // Deactivate attributes that no longer exist
            $sql = "UPDATE {$dbp}sniperfast_filters_position
                SET active = 0
                WHERE id_shop = " . (int) $idShop .
                ' AND id_lang = ' . (int) $idLang .
                " AND type = 'attr'
                AND value NOT IN (
                SELECT id_attribute_group
                FROM {$dbp}attribute_group
            )";
            $db->execute($sql);
        }
    }

    public static function setConfig($key, $value, $idShop, $idLang)
    {
        if ($key == null || $key == '') {
            return false;
        }

        $table = _DB_PREFIX_ . 'sniperfast_conf';
        $key = pSQL($key);
        $value = pSQL($value);
        $idShop = (int) $idShop;
        $idLang = (int) $idLang;

        // All the variables in this query are already casted right above
        $sql = "INSERT INTO $table
            VALUES ('$key', $idShop, $idLang, '$value')
            ON DUPLICATE KEY
            UPDATE conf_value = '$value'";

        return Db::getInstance()->execute($sql);
    }

    public static function setAppConfig($key, $value, $idShop)
    {
        if ($key == null || $key == '') {
            return false;
        }

        $table = _DB_PREFIX_ . 'sniperfast_app';
        $key = pSQL($key);
        $value = pSQL($value);
        $idShop = (int) $idShop;

        // All the variables in this query are already casted right above
        $sql = "INSERT INTO $table
            VALUES ($idShop, '$key', '$value')
            ON DUPLICATE KEY
            UPDATE conf_value = '$value'";

        return Db::getInstance()->execute($sql);
    }

    public static function getConfig($key, $idShop, $idLang)
    {
        if (empty($key) || empty($idShop) || empty($idLang)) {
            return false;
        }

        $table = _DB_PREFIX_ . 'sniperfast_conf';
        $key = pSQL($key);
        $idShop = (int) $idShop;
        $idLang = (int) $idLang;

        // All the variables in this query are already casted right above
        $sql = "SELECT conf_value
            FROM $table
            WHERE conf_key = '$key'
            AND id_shop = $idShop
            AND id_lang = $idLang";
        $res = Db::getInstance()->executeS($sql);
        if (empty($res)) {
            return '';
        }

        return $res[0]['conf_value'];
    }

    public static function getAppConfig($key, $idShop)
    {
        if (empty($key) || empty($idShop)) {
            return false;
        }

        $table = _DB_PREFIX_ . 'sniperfast_app';
        $key = pSQL($key);
        $idShop = (int) $idShop;

        // All the variables in this query are already casted right above
        $sql = "SELECT conf_value
            FROM $table
            WHERE conf_key = '$key'
            AND id_shop = $idShop";
        $res = Db::getInstance()->executeS($sql);
        if (empty($res)) {
            return '';
        }

        return $res[0]['conf_value'];
    }

    public static function generateNewAjaxToken($idShop, $idLang, $tokenType)
    {
        $token = bin2hex(openssl_random_pseudo_bytes(16));

        return self::setConfig($tokenType, $token, $idShop, $idLang);
    }

    public static function cmsPageExists($dedicatedPageId, $idShop)
    {
        $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'cms
            WHERE id_cms = ' . (int) $dedicatedPageId;
        $cmsRes = Db::getInstance()->executeS($sql);
        if (empty($cmsRes)) {
            $sql = 'UPDATE ' . _DB_PREFIX_ . "sniperfast_conf
                SET conf_value = 0
                WHERE conf_key = 'dedi_page_id'
                AND id_shop = " . (int) $idShop;
            Db::getInstance()->execute($sql);

            return false;
        }

        return true;
    }

    public static function updateMainData($idShop, $idLang, &$errorMsg)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, MANAGER_ENDPOINT . '/sniperfast/getconfiguration');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, 10);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query([
            'token' => SniperCommon::getConfig('migration_token', $idShop, 200000),
            'index_name' => SniperCommon::getConfig('sniper_index_name', $idShop, $idLang),
        ]));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $res = curl_exec($ch);

        if (curl_errno($ch)) {
            $errorMsg = curl_error($ch);
            curl_close($ch);

            return false;
        }

        curl_close($ch);

        if (empty($res)) {
            $errorMsg = 'connection_error';

            return false;
        }

        $responseData = json_decode($res, true);

        if (!isset($responseData['success']) || !$responseData['success']) {
            $errorMsg = 'invalid_data';

            return false;
        }

        $savedSettings = $responseData['result'];
        $sniperSettings = new SniperfastSettings($idShop, $idLang);

        $bannerDesktop = [];
        if (isset($savedSettings['banner_desktop'])) {
            $bannerDesktop = $savedSettings['banner_desktop'];
        }
        $bannerMobile = [];
        if (isset($savedSettings['banner_mobile'])) {
            $bannerMobile = $savedSettings['banner_mobile'];
        }

        try {
            SniperCommon::setConfig('remarketing_token', $savedSettings['remarketing_token'], $idShop, $idLang);
            $sniperSettings->updateMainDataFromManager($savedSettings, $bannerDesktop, $bannerMobile, $savedSettings['remarketing_token']);
        } catch (Exception $e) {
            $errorMsg = 'settings_error';

            return false;
        }

        return true;
    }

    public static function getDedicatedPageUrl($idShop, $idLang)
    {
        $dedicatedPageUrl = null;
        if (SniperCommon::getConfig('dedi_page_enable', $idShop, $idLang)) {
            $dedicatedPageId = SniperCommon::getConfig('dedi_page_id', $idShop, $idLang);
            $cmsPage = new CMS((int) $dedicatedPageId);
            $link = new Link();
            $dedicatedPageUrl = $link->getCMSLink($cmsPage, null, null, $idLang);
        }

        return $dedicatedPageUrl;
    }

    public static function getTagPageUrl($idShop, $idLang)
    {
        $tagPageUrl = null;
        if (SniperCommon::getConfig('tag_page_enable', $idShop, $idLang)) {
            $tagPageId = SniperCommon::getConfig('tag_page_id', $idShop, $idLang);
            $cmsPage = new CMS((int) $tagPageId);
            $link = new Link();
            $tagPageUrl = $link->getCMSLink($cmsPage, null, null, $idLang);
        }

        return $tagPageUrl;
    }

    public static function addProductToQueue($idProduct, $time, $idShop)
    {
        $sql = 'INSERT INTO ' . _DB_PREFIX_ . 'sniperfast_queue
            (id_product, id_shop, updated_on) VALUES (' .
                (int) $idProduct . ', ' .
                (int) $idShop . ', ' .
                (int) $time .
            ') ON DUPLICATE KEY UPDATE updated_on = ' . (int) $time;
        try {
            return Db::getInstance()->execute($sql);
        } catch (Exception $e) {
            // Do nothing
        }
    }

    public static function getLastIncrementalUpdate($idShop)
    {
        $sql = 'SELECT conf_value FROM ' . _DB_PREFIX_ . "sniperfast_conf
            WHERE conf_key = 'last_incremental_update'
            AND id_shop = " . (int) $idShop .
            ' AND id_lang = 131000';
        $lastUpdate = Db::getInstance()->getValue($sql);
        if (empty($lastUpdate)) {
            return '0_0';
        } else {
            return $lastUpdate;
        }
    }

    public static function setLastIncrementalUpdate($currentTime, $idProduct, $idShop)
    {
        self::setConfig('last_incremental_update', $idProduct . '_' . $currentTime, $idShop, 131000);
    }

    public static function getQueuedProducts($idShop)
    {
        $sql = 'SELECT id_product FROM ' . _DB_PREFIX_ . 'sniperfast_queue
            WHERE id_shop = ' . (int) $idShop;
        $res = Db::getInstance()->executeS($sql);
        if (empty($res)) {
            return [];
        }

        $queuedProducts = [];
        foreach ($res as $r) {
            $queuedProducts[] = $r['id_product'];
        }

        return $queuedProducts;
    }

    public static function getConfigKeys($idShop, $idLang)
    {
        $configKeys = [
            'exports_gap' => 'int',
            'sniper_features' => 'string',
            'sniper_attributes' => 'string',
            'image_format' => 'int',
            'only_in_stock' => 'int',
            'remove_vat' => 'int',
            'available_filter' => 'int',
            'available_filter_name' => 'string',
            'available_filter_instock' => 'string',
            'available_filter_outstock' => 'string',
            'sniper_currency' => 'int',
            'export_batch_size' => 'int',
            'sniper_free_ship' => 'float',
            'sniper_export_review' => 'int',
        ];
        $config = [];

        foreach ($configKeys as $key => $type) {
            $config[$key] = self::getConfig($key, $idShop, $idLang);

            if ($key == 'export_batch_size' && $config[$key] == '') {
                $config[$key] = 300;
                continue;
            }

            if ($config[$key] == ''
                && ($key == 'sniper_free_ship'
                    || $key == 'sniper_export_review'
                    || $key == 'available_filter'
                    || $key == 'only_in_stock'
                    || $key == 'sniper_currency'
                    || $key == 'remove_vat'
                )
            ) {
                $config[$key] = 0;
                continue;
            }

            // If image format is not configured (i.e. on boarding), then we get one for products (possibly cart one).
            if ($config[$key] == '' && $key == 'image_format') {
                $out = [];
                $outCart = [];
                $psImgTypes = ImageType::getImagesTypes(null, true);

                foreach ($psImgTypes as $type) {
                    if ($type['products'] != 1) {
                        continue;
                    }
                    $out[] = $type['id_image_type'];

                    if (Tools::strpos($type['name'], 'cart') !== false) {
                        $outCart[] = $type['id_image_type'];
                    }
                }

                if (!empty($outCart)) {
                    $config[$key] = array_shift($outCart);
                    continue;
                } elseif (!empty($out)) {
                    $config[$key] = array_shift($outCart);
                    continue;
                }
            }

            if ($config[$key] == '') {
                if ($key == 'sniper_features' || $key == 'sniper_attributes') {
                    continue;
                }

                return [];
            }

            // Cast config value to the proper type
            settype($config[$key], $type);
        }

        $imgTypes = ImageType::getImagesTypes();

        foreach ($imgTypes as $imgType) {
            if ($config['image_format'] == $imgType['id_image_type']) {
                $config['img_type'] = $imgType['name'];
                break;
            }
        }

        return $config;
    }

    public static function sendOrderDetails($idShop, $idLang, $orderData)
    {
        if (!self::getConfig('sniper_index_name', $idShop, $idLang)) {
            return;
        }
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, CDN_PROTOCOL . '://api.sniperfast.' . CDN_ENDPOINT . '/stats?type=send_order');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query([
            'shop' => $idShop,
            'lang' => $idLang,
            'key' => self::getConfig('sniper_key', $idShop, $idLang),
            'index' => self::getConfig('sniper_index_name', $idShop, $idLang),
            'order_data' => $orderData,
            'ip' => $_SERVER['HTTP_X_REAL_IP'],
        ]));

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_exec($ch);
        curl_close($ch);
    }

    public static function updatePositions($idElement, $type, $idShop, $idLang)
    {
        $table = '';
        if ($type == 'filter') {
            $table = 'sniperfast_filters_position';
        } elseif ($type == 'relevance') {
            $table = 'sniperfast_relevance_position';
        }

        $position = 0;
        $db = Db::getInstance();
        foreach ($idElement as $el) {
            $el = explode('_', $el);
            array_pop($el);
            $id = array_pop($el);

            $sql = 'UPDATE ' . _DB_PREFIX_ . pSQL($table) .
            ' SET position = ' . (int) $position .
            ' WHERE id_' . pSQL($type) . ' = ' . (int) $id .
            ' AND id_shop = ' . (int) $idShop .
            ' AND id_lang = ' . (int) $idLang;
            $db->execute($sql);
            ++$position;
        }

        return true;
    }

    public static function getMostSearchedTerms($idShop, $idLang)
    {
        if (!self::getConfig('sniper_index_name', $idShop, $idLang)) {
            return;
        }
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, CDN_PROTOCOL . '://api.sniperfast.' . CDN_ENDPOINT . '/stats?type=search_terms');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query([
            'key' => self::getConfig('sniper_key', $idShop, $idLang),
            'index' => self::getConfig('sniper_index_name', $idShop, $idLang),
            'country_iso' => '',
            'is_app' => 0,
            'excluded_ips' => '',
        ]));

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $res = curl_exec($ch);
        curl_close($ch);

        if (empty($res)) {
            return [];
        }

        $res = json_decode($res, true);
        $mostSearchedTerms = [];
        if (isset($res['data']) && !empty($res['data']['searches'])) {
            foreach ($res['data']['searches'] as $r) {
                $mostSearchedTerms[] = $r['key'];
            }
        }

        return $mostSearchedTerms;
    }

    public static function initRelevancePosition($idShop, $idLang)
    {
        $sql = 'INSERT IGNORE INTO ' .
            _DB_PREFIX_ . 'sniperfast_relevance_position(id_shop, id_lang, type, position) VALUES ';
        $types = [
            'rel',
            'best',
            'name_asc',
            'name_dec',
            'price_asc',
            'price_dec',
            'qty_asc',
            'qty_dec',
            'reference_asc',
            'reference_dec',
            'date_asc',
            'date_dec',
        ];

        $i = 0;
        $sqlValues = [];
        foreach ($types as $t) {
            $tempVal = [
                (int) $idShop,
                (int) $idLang,
                "'" . pSQL($t) . "'",
                (int) $i,
            ];
            $sqlValues[] = '(' . implode(',', $tempVal) . ')';
            ++$i;
        }
        try {
            Db::getInstance()->execute($sql . implode(',', $sqlValues));
        } catch (Exception $e) {
        }
    }

    public static function getFirstSuperadminEmployeeId()
    {
        $sql = 'SELECT `id_employee` FROM `' . _DB_PREFIX_ . 'employee`
            WHERE `id_profile` = 1
            ORDER BY `id_employee`
            LIMIT 1';

        $result = Db::getInstance()->getValue($sql);

        return ($result !== false) ? (int) $result : 1;
    }

    public static function getDaysFromExpiration($expTime, $currentTime, &$expirationFormatted)
    {
        $expiryDate = DateTime::createFromFormat('U.u', number_format($expTime, 6, '.', ''));
        $currentTimeDt = DateTime::createFromFormat('U.u', number_format($currentTime, 6, '.', ''));
        $expirationFormatted = $expiryDate->format('d m Y');

        // < 0 if expiration is in the future, > 0 if subscription already expired
        return $expiryDate->diff($currentTimeDt)->format('%r%a');
    }

    public static function expirationCheckRequired($days, $lastTimeCheck, $currentTimestamp, $isExpired)
    {
        $checkRequired = false;
        if ($days < -5 || $isExpired) {
            // Check if a week has passed
            if ($currentTimestamp - $lastTimeCheck > (7 * 24 * 60 * 60)) {
                $checkRequired = true;
            }
        } elseif ($days >= -5 && $days <= 0) {
            // Check if a day has passed
            if ($currentTimestamp - $lastTimeCheck > (24 * 60 * 60)) {
                $checkRequired = true;
            }
        }

        return $checkRequired;
    }
}
