<?php
/**
 *  Copyright (C) Prestalia - All Rights Reserved
 *  Unauthorized copying of this file, via any medium is strictly prohibited
 *  Proprietary and confidential
 *
 *  @author    Prestalia <prestalia.it>
 *  @copyright 2019 - 2024 Prestalia
 *  @license   Commercial
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once dirname(__FILE__) . '/../autoload.php';

class SniperUpgradeHelper
{
    private $idShop;
    private $languages;

    public function __construct($idShop)
    {
        $this->idShop = $idShop;
        $this->languages = $this->getWebsiteLanguages($idShop);
    }

    public function convertDiv()
    {
        foreach ($this->languages as $lang) {
            $sniperDiv = SniperCommon::getConfig('sniper_div_id', $this->idShop, $lang['id']);
            if (empty($sniperDiv)) {
                continue;
            }

            $sniperDiv = explode(',', $sniperDiv);
            if (isset($sniperDiv[0])) {
                SniperCommon::setConfig('sniper_div_dsk', $sniperDiv[0], $this->idShop, $lang['id']);
            }
            if (isset($sniperDiv[1])) {
                SniperCommon::setConfig('sniper_div_mob', $sniperDiv[1], $this->idShop, $lang['id']);
            }
        }
    }

    public function updateMainFile()
    {
        $path = _PS_MODULE_DIR_ . 'sniperfast';
        foreach ($this->languages as $lang) {
            $sourceFile = '';
            if ($lang['iso_code'] == 'it') {
                $sourceFile = $path . '/views/js/main_it.js';
            } else {
                $sourceFile = $path . '/views/js/main_en.js';
            }

            if (!file_exists($sourceFile)) {
                continue;
            }

            $sniperIndex = SniperCommon::getConfig('sniper_index_name', $this->idShop, $lang['id']);
            if (empty($sniperIndex)) {
                continue;
            }
            $submittedValues = [];
            $template = Tools::file_get_contents($sourceFile);
            if ($template === false) {
                continue;
            }

            $template = str_replace(
                array_keys($submittedValues),
                $submittedValues,
                $template
            );

            $date = new DateTime();
            $dateFormat = $date->format('Y-m-d_H-i-s');
            $newTemplateName = md5($dateFormat) . $lang['id'];
            $newTemplate = $path . '/views/js/' . $newTemplateName . '.js';
            if (!file_put_contents($newTemplate, $template)) {
                continue;
            } else {
                $oldFile = SniperCommon::getConfig('main_data_file', $this->idShop, $lang['id']);
                $oldFilePath = $path . '/views/js/' . $oldFile . '.js';
                if (file_exists($oldFilePath)) {
                    unlink($oldFilePath);
                }
                SniperCommon::setConfig('main_data_file', $newTemplateName, $this->idShop, $lang['id']);
            }
        }
    }

    protected function getWebsiteLanguages($idShop)
    {
        $languages = Language::getLanguages(true, $idShop);
        $languageForMenu = [];

        foreach ($languages as $lang) {
            $l = [];
            $l['id'] = $lang['id_lang'];
            $l['iso_code'] = $lang['iso_code'];
            $languageForMenu[] = $l;
        }

        return $languageForMenu;
    }

    protected function getDedicatedPageUrl($idShop, $idLang)
    {
        $dedicatedPageEnable = SniperCommon::getConfig('dedi_page_enable', $idShop, $idLang);
        if (!$dedicatedPageEnable) {
            return null;
        }

        $dedicatedPageId = SniperCommon::getConfig('dedi_page_id', $idShop, $idLang);
        if (empty($dedicatedPageId)) {
            return null;
        }

        if (!empty($dedicatedPageId) && !SniperCommon::cmsPageExists($dedicatedPageId, $idShop)) {
            return null;
        }

        $cmsPage = new CMS((int) $dedicatedPageId);
        $link = new Link();

        return $link->getCMSLink($cmsPage);
    }
}
