<?php
/**
 *  Copyright (C) Prestalia - All Rights Reserved
 *  Unauthorized copying of this file, via any medium is strictly prohibited
 *  Proprietary and confidential
 *
 *  @author    Prestalia <prestalia.it>
 *  @copyright 2019 - 2024 Prestalia
 *  @license   Commercial
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class SniperfastLandingPage
{
    private $idShop;
    private $idLang;
    private $idLanding;

    public function __construct($idShop, $idLang, $idLanding)
    {
        $this->idShop = $idShop;
        $this->idLang = $idLang;
        $this->idLanding = $idLanding;
    }

    public function getLandingPageConfiguration()
    {
        $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'sniperfast_landing
            WHERE id_shop = ' . (int) $this->idShop .
            ' AND id_lang = ' . (int) $this->idLang .
            ' AND id_landing = ' . (int) $this->idLanding;
        $res = Db::getInstance()->executeS($sql);
        $landingConfiguration = [];
        if (!empty($res)) {
            $landingConfiguration = array_pop($res);
        }

        return $landingConfiguration;
    }

    public function getCategoriesPath($landingConfiguration)
    {
        $categorySearchPath = [
            'cat1' => '',
            'cat2' => '',
            'cat3' => '',
            'cat4' => '',
        ];
        $idCategory = 0;
        if ($landingConfiguration['source'] == 2 && !empty($landingConfiguration['source_data'])) {
            $idCategory = (int) $landingConfiguration['source_data'];
        } else {
            return $categorySearchPath;
        }
        $path = Tools::getPath('', $idCategory);
        $path = strip_tags($path);
        $path = str_replace(Configuration::get('PS_NAVIGATION_PIPE'), ' > ', $path);
        $explodedPath = explode(' > ', $path);
        if (count($explodedPath) > 1) {
            $explodedPath = array_slice($explodedPath, 1);
        }

        $explodedPath = array_filter(array_map('trim', $explodedPath));
        if (!empty($explodedPath)) {
            $count = 1;
            foreach ($explodedPath as $ep) {
                if ($count > 4) {
                    break;
                }
                $categorySearchPath['cat' . $count] = $ep;
                ++$count;
            }
        }

        return $categorySearchPath;
    }

    public function getManufacturerName($landingConfiguration)
    {
        if ($landingConfiguration['source'] == 3 && !empty($landingConfiguration['source_data'])) {
            $params = SniperCommonApp::getManufacturerForQuery($landingConfiguration['source_data']);

            return $params['snmanu'];
        } else {
            return '';
        }
    }

    public function getLandingPageFormData()
    {
        $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'sniperfast_landing
            WHERE id_landing = ' . (int) $this->idLanding;
        $res = Db::getInstance()->executeS($sql);

        $landingData = [];
        if (empty($res)) {
            return $landingData;
        }

        foreach ($res as $r) {
            foreach ($r as $key => $val) {
                $landingData[$key][$r['id_lang']] = $val;
            }
        }

        return $landingData;
    }

    public static function getLandingPageTranslatableFields()
    {
        $translatableValues = [
            'landing_title' => '',
            'landing_rewrite' => '',
            'landing_description' => '',
            'landing_title_meta' => '',
            'landing_description_meta' => '',
            'landing_page_image' => '',
            'landing_search_query' => '',
        ];

        return $translatableValues;
    }

    public static function setLandingPageData(
        $idLanding,
        $idShop,
        $idLang,
        $landingSource,
        $landingSourceData,
        $landingData
    ) {
        $dbp = _DB_PREFIX_;
        $db = Db::getInstance();
        $cleanData = SniperCommonApp::getCleanData($landingSourceData);

        if (empty($landingData['landing_title'])) {
            $landingData['landing_title'] = 'Landing page';
        }

        if (empty($landingData['landing_rewrite'])) {
            $landingData['landing_rewrite'] = 'landing-page';
        }

        $sql = "INSERT INTO {$dbp}sniperfast_landing
            (id_landing, id_shop, id_lang, name, link_rewrite, description, image, meta_title,
                meta_description, source, source_data, search_query
            ) VALUES (" .
                (int) $idLanding . ', ' .
                (int) $idShop . ', ' .
                (int) $idLang . ', ' .
                "'" . pSQL($landingData['landing_title']) . "', " .
                "'" . pSQL($landingData['landing_rewrite']) . "', " .
                "'" . pSQL($landingData['landing_description']) . "', " .
                "'" . Tools::purifyHTML($landingData['landing_page_image'], null, true) . "', " .
                "'" . pSQL($landingData['landing_title_meta']) . "', " .
                "'" . pSQL($landingData['landing_description_meta']) . "', " .
                (int) $landingSource . ', ' .
                $cleanData . ', ' .
                "'" . pSQL($landingData['landing_search_query']) . "'" .
            ") ON DUPLICATE KEY UPDATE
                name = '" . pSQL($landingData['landing_title']) . "', " .
                "link_rewrite = '" . pSQL($landingData['landing_rewrite']) . "', " .
                "description = '" . pSQL($landingData['landing_description']) . "', " .
                "image = '" . Tools::purifyHTML($landingData['landing_page_image'], null, true) . "', " .
                "meta_title = '" . pSQL($landingData['landing_title_meta']) . "', " .
                "meta_description = '" . pSQL($landingData['landing_description_meta']) . "', " .
                'source = ' . (int) $landingSource . ', ' .
                'source_data = ' . $cleanData . ', ' .
                "search_query = '" . pSQL($landingData['landing_search_query']) . "'";

        $db->execute($sql);
    }

    public static function getAllLandingPageForLanguage($idShop, $idLang)
    {
        $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'sniperfast_landing
            WHERE id_shop = ' . (int) $idShop .
            ' AND id_lang = ' . (int) $idLang;

        return Db::getInstance()->executeS($sql);
    }
}
