<?php
/**
 *  Copyright (C) Prestalia - All Rights Reserved
 *  Unauthorized copying of this file, via any medium is strictly prohibited
 *  Proprietary and confidential
 *
 *  @author    Prestalia <prestalia.it>
 *  @copyright 2019 - 2024 Prestalia
 *  @license   Commercial
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class SniperfastSettings
{
    private $idShop;
    private $idLang;
    private $db;
    private $dbp;

    public function __construct($idShop, $idLang)
    {
        $this->idShop = (int) $idShop;
        $this->idLang = (int) $idLang;
        $this->db = Db::getInstance();
        $this->dbp = _DB_PREFIX_;
    }

    public function registerRecoverAccount($token, $type, &$msg)
    {
        $storeAddress = $this->getStoreAddress();
        $baseUrl = $this->getBaseUrl();
        $shopName = str_replace(['http://', 'https://'], '', $baseUrl);
        $shopName = preg_replace('/[^a-z\d]+/i', '', $shopName);
        $shopName = Tools::substr($shopName, 0, 52);
        $context = Context::getContext();
        $params = [
            'token' => $token,
            'shop_id' => $this->idShop,
            'shop_name' => $shopName,
            'shop_url' => $baseUrl,
            'shop_email' => Configuration::get('PS_SHOP_EMAIL'),
            'shop_address' => json_encode($storeAddress),
            'type' => 1,
            'lang' => $this->idLang,
            'lang_iso' => Language::getIsoById($this->idLang),
            'shop_ip' => isset($_SERVER['SERVER_ADDR']) ? $_SERVER['SERVER_ADDR'] : 'localhost',
            'ajax_token_remote' => SniperCommon::getConfig('manager_token', $this->idShop, $this->idLang),
            'controller_url' => $context->link->getModuleLink('sniperfast', 'SniperData', []),
            'filters_position' => $this->getFiltersPosition(),
            'relevance_position' => $this->getRelevancePosition(),
            'configuration' => $this->getConf(),
            'synonyms' => file_exists(_PS_MODULE_DIR_ . 'sniperfast/output/' . $this->idShop . '_' . $this->idLang . '_synonims.txt') ? Tools::file_get_contents(_PS_MODULE_DIR_ . 'sniperfast/output/' . $this->idShop . '_' . $this->idLang . '_synonims.txt') : '',
            'css_custom' => file_exists(_PS_MODULE_DIR_ . 'sniperfast/views/css/' . $this->idShop . '_' . $this->idLang . '_custom.css') ? Tools::file_get_contents(_PS_MODULE_DIR_ . 'sniperfast/views/css/' . $this->idShop . '_' . $this->idLang . '_custom.css') : '',
        ];

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, MANAGER_ENDPOINT . '/sniperfast/registermodule');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));
        curl_setopt($ch, CURLOPT_HEADER, false);
        // curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 30);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        try {
            $res = curl_exec($ch);
            curl_close($ch);
            $res = json_decode($res, true);
            if (empty($res)) {
                $msg = 'autoregister_error';

                return false;
            }
            if (!isset($res['success']) || !$res['success']) {
                $msg = 'unexpected_error';
                if (isset($res['msg'])) {
                    $msg = $res['msg'];
                }

                return false;
            }

            $sub = $res['subscription'];
            SniperCommon::setConfig('sniper_index_name', $sub['index_name'], $this->idShop, $this->idLang);
            SniperCommon::setConfig('sniper_key', $sub['index_key'], $this->idShop, $this->idLang);
            SniperCommon::setConfig('migration_token', $token, $this->idShop, 200000);
            // SniperCommon::setConfig('enable_search', 0, $this->idShop, $this->idLang);
            if ($type == 'active' || $type == 'full') {
                SniperCommon::updateMainData($this->idShop, $this->idLang, $msg);
            }

            $msg = $res['msg'];

            return true;
        } catch (Exception $e) {
            $msg = $e->getMessage();

            return false;
        }
    }

    public static function getMigrationToken($email, $password)
    {
        $params = [
            'email' => $email,
            'password' => $password,
        ];

        $url = MANAGER_ENDPOINT . '/sniperfast/getmigrationtoken';

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, MANAGER_ENDPOINT . '/sniperfast/getmigrationtoken');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        try {
            $res = curl_exec($ch);
            curl_close($ch);
            $res = json_decode($res, true);

            if ($res['success'] !== true || !isset($res['token'])) {
                return false;
            } else {
                return $res['token'];
            }
        } catch (Exception $e) {
            $res = [];

            return false;
        }
    }

    private function getFiltersPosition()
    {
        $sql = "SELECT `type`, `value`, '' AS `name`,  `has_search`, `is_open`, `position`, `active`
            FROM {$this->dbp}sniperfast_filters_position
            WHERE id_shop = " . (int) $this->idShop .
            ' AND id_lang = ' . (int) $this->idLang;
        $resFilter = $this->db->executeS($sql);

        if (empty($resFilter)) {
            return [];
        }

        $feat = [];
        $attr = [];

        foreach ($resFilter as $r) {
            if ($r['type'] == 'feat') {
                if ($r['value'] == 9950) {
                    continue;
                }
                $feat[] = $r['value'];
            } elseif ($r['type'] == 'attr') {
                $attr[] = $r['value'];
            }
        }

        $featData = [];
        if (!empty($feat)) {
            $sql = "SELECT id_feature, `name`
                FROM {$this->dbp}feature_lang
                WHERE id_feature IN (" . implode(',', array_map('intval', $feat)) . ')
                AND id_lang = ' . (int) $this->idLang;
            $resFeat = $this->db->executeS($sql);
            foreach ($resFeat as $r) {
                $featData[$r['id_feature']] = $r['name'];
            }
        }
        $attrData = [];
        if (!empty($attr)) {
            $sql = "SELECT id_attribute_group, `name`
                FROM {$this->dbp}attribute_group_lang
                WHERE id_attribute_group IN (" . implode(',', array_map('intval', $attr)) . ')
                AND id_lang = ' . (int) $this->idLang;
            $resAttr = $this->db->executeS($sql);
            $attrData = [];
            foreach ($resAttr as $r) {
                $attrData[$r['id_attribute_group']] = $r['name'];
            }
        }

        foreach ($resFilter as &$rf) {
            if ($rf['type'] == 'feat' && isset($featData[$rf['value']])) {
                $rf['name'] = $featData[$rf['value']];
            } elseif ($rf['type'] == 'attr' && isset($attrData[$rf['value']])) {
                $rf['name'] = $attrData[$rf['value']];
            }
        }
        unset($rf);

        return $resFilter;
    }

    private function getRelevancePosition()
    {
        $sql = "SELECT `type`, `position`, `active`
            FROM {$this->dbp}sniperfast_relevance_position
            WHERE id_shop = " . (int) $this->idShop .
            ' AND id_lang = ' . (int) $this->idLang;
        $res = $this->db->executeS($sql);

        if (empty($res)) {
            return [];
        }

        return $res;
    }

    private function getConf()
    {
        $sql = "SELECT conf_key, conf_value
            FROM {$this->dbp}sniperfast_conf
            WHERE id_shop = " . (int) $this->idShop .
            ' AND id_lang = ' . (int) $this->idLang;
        $res = $this->db->executeS($sql);

        $conf = [
            'sniper_index_name' => '',
            'sniper_key' => '',
        ];
        if (!empty($res)) { // It should never happen
            foreach ($res as $r) {
                $conf[$r['conf_key']] = $r['conf_value'];
            }
        }

        return $conf;
    }

    private function getStoreAddress()
    {
        $storeAddress = [];
        if (Tools::version_compare(_PS_VERSION_, '1.7.3.0', '<')) {
            $sql = "SELECT so.address1, so.address2, so.city, so.postcode AS zip,
                so.phone, cl.name AS country
                FROM {$this->dbp}store AS so
                LEFT JOIN {$this->dbp}store_shop AS ss ON so.id_store = ss.id_store
                LEFT JOIN {$this->dbp}country_lang AS cl ON so.id_country = cl.id_country
                WHERE ss.id_shop = " . (int) $this->idShop .
                ' AND cl.id_lang = ' . (int) $this->idLang .
                ' LIMIT 1';
            $storeAddress = $this->db->executeS($sql);
        } else {
            $sql = "SELECT so.city, sl.address1, sl.address2, so.postcode AS zip,
                so.phone, cl.name AS country
                FROM {$this->dbp}store AS so
                LEFT JOIN {$this->dbp}store_shop AS ss ON so.id_store = ss.id_store
                LEFT JOIN {$this->dbp}store_lang AS sl ON so.id_store = sl.id_store
                LEFT JOIN {$this->dbp}country_lang AS cl ON so.id_country = cl.id_country
                WHERE ss.id_shop = " . (int) $this->idShop .
                ' AND sl.id_lang = ' . (int) $this->idLang .
                ' AND cl.id_lang = ' . (int) $this->idLang .
                ' LIMIT 1';
            $storeAddress = $this->db->executeS($sql);
        }
        if (empty($storeAddress)) {
            return [];
        }

        return $storeAddress[0];
    }

    public function saveFeatureRange($range, &$msg)
    {
        $msg = 'feature_success';
        $featureRangeFile = dirname(__FILE__) . '/../output/' . $this->idShop . '_' . $this->idLang . '_range_feature.json';
        $errorMsg = '';
        if (!file_exists($featureRangeFile) && empty($range)) {
            return true;
        } elseif (!file_exists($featureRangeFile) && !empty($range)) {
            file_put_contents($featureRangeFile, $range);
            SniperCommon::updateMainData($this->idShop, $this->idLang, $errorMsg);

            return true;
        } else {
            $previousRange = Tools::file_get_contents($featureRangeFile);
            if ($previousRange == $range['features_details']) {
                return true;
            }
            file_put_contents($featureRangeFile, $range);
            SniperCommon::updateMainData($this->idShop, $this->idLang, $errorMsg);

            return true;
        }
    }

    public function saveJobId($idJob, &$msg)
    {
        if (!is_numeric($idJob)) {
            $msg = 'invalid_id_job';

            return false;
        }
        SniperCommon::setConfig('last_job_id', $idJob, $this->idShop, $this->idLang);
        $msg = 'id_job_save_success';

        return true;
    }

    public function deleteIndexAndKey(&$msg)
    {
        $sql = "DELETE FROM {$this->dbp}sniperfast_conf
            WHERE conf_key IN ('sniper_index_name', 'sniper_key')
            AND id_shop = " . (int) $this->idShop .
            ' AND id_lang = ' . (int) $this->idLang;
        $this->db->execute($sql);

        $msg = 'delete_success';

        return true;
    }

    public function convertIndex(&$msg)
    {
        $shopUrl = $this->getBaseUrl();
        $indexName = SniperCommon::getConfig('sniper_index_name', $this->idShop, $this->idLang);
        $indexKey = SniperCommon::getConfig('sniper_key', $this->idShop, $this->idLang);
        $shopName = str_replace(['http://', 'https://'], '', $shopUrl);
        $shopName = preg_replace('/[^a-z\d]+/i', '', $shopName);
        $shopName = Tools::substr($shopName, 0, 52);

        $params = [
            'shop_id' => $this->idShop,
            'shop_url' => $shopUrl,
            'shop_name' => $shopName,
            'lang' => $this->idLang,
            'index_name' => $indexName,
            'index_key' => $indexKey,
            'shop_ip' => $_SERVER['SERVER_ADDR'],
        ];

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, CDN_PROTOCOL . '://index.sniperfast.' . CDN_ENDPOINT . '/convertindex');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, 15);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $res = curl_exec($ch);
        curl_close($ch);
        $res = json_decode($res, true);

        if (empty($res)) {
            $msg = 'conversion_error';

            return false;
        }

        if (!$res['success']) {
            $msg = $res['msg'];

            return false;
        }
        SniperCommon::setConfig('sniper_index_name', $res['sniper_index'], $this->idShop, $this->idLang);
        SniperCommon::setConfig('sniper_key', $res['sniper_key'], $this->idShop, $this->idLang);
        SniperCommon::updateMainData($this->idShop, $this->idLang, $msg);
        $msg = $res['msg'];

        return true;
    }

    public function flipStatus($id, $type, &$msg)
    {
        if (empty($id) || $id < 0) {
            $msg = 'invalid_id';

            return false;
        }
        $id = (int) $id;
        $msg = 'update_error';
        $success = false;
        switch ($type) {
            case 'category':
                $filter = new SniperfastProductFilter($this->idShop, $this->idLang);
                $isEnabled = $filter->isCategoryEnabled($id);
                if ($isEnabled === null) {
                    $isEnabled = $this->getEnabledFromDb($id, $type);
                    if ($isEnabled === false) {
                        $isEnabled = 1;
                    }
                }
                if (!$isEnabled) {
                    $filter->enableCategory($id);
                } else {
                    $filter->disableCategory($id);
                }
                $msg = 'update_success';
                $success = true;
                break;
            case 'filter':
                $resUpdate = false;
                $res = $this->flipFilter($id);
                if ($res) {
                    /* Not ideal, but this operation usually is performed once and stop,
                     * as such we update js everytime. We can't rely on user saving data
                     * in the General tab.
                     */
                    $resUpdate = SniperCommon::updateMainData($this->idShop, $this->idLang, $msg);
                    if ($resUpdate) {
                        $msg = 'update_success';
                    }
                }
                $success = $res && $resUpdate;
                break;
            case 'manufacturer':
                $filter = new SniperfastProductFilter($this->idShop, $this->idLang);
                $isEnabled = $filter->isManufacturerEnabled($id);
                if ($isEnabled === null) {
                    $isEnabled = $this->getEnabledFromDb($id, $type);
                    if ($isEnabled === false) {
                        $isEnabled = 1;
                    }
                }
                if (!$isEnabled) {
                    $filter->enableManufacturer($id);
                } else {
                    $filter->disableManufacturer($id);
                }
                $msg = 'update_success';
                $success = true;
                break;
            case 'open':
                $resUpdate = false;
                $res = $this->flipOpen($id);
                if ($res) {
                    /* Not ideal, but this operation usually is performed once and stop,
                     * as such we update js everytime. We can't rely on user saving data
                     * in the General tab.
                     */
                    $resUpdate = SniperCommon::updateMainData($this->idShop, $this->idLang, $msg);
                    if ($resUpdate) {
                        $msg = 'update_success';
                    }
                }
                $success = $res && $resUpdate;
                break;
            case 'product':
                $filter = new SniperfastProductFilter($this->idShop, $this->idLang);
                if (!$filter->isProductEnabled($id)) {
                    $filter->enableProduct($id);
                } else {
                    $filter->disableProduct($id);
                }
                $msg = 'update_success';
                $success = true;
                break;
            case 'supplier':
                $filter = new SniperfastProductFilter($this->idShop, $this->idLang);
                $isEnabled = $filter->isSupplierEnabled($id);
                if ($isEnabled === null) {
                    $isEnabled = $this->getEnabledFromDb($id, $type);
                    if ($isEnabled === false) {
                        $isEnabled = 1;
                    }
                }
                if (!$isEnabled) {
                    $filter->enableSupplier($id);
                } else {
                    $filter->disableSupplier($id);
                }
                $msg = 'update_success';
                $success = true;
                break;
        }

        return $success;
    }

    private function flipFilter($id)
    {
        $sql = "UPDATE {$this->dbp}sniperfast_filters_position
            SET `has_search` = NOT `has_search`
            WHERE id_filter = " . (int) $id;

        return $this->db->execute($sql);
    }

    private function flipOpen($id)
    {
        $sql = "UPDATE {$this->dbp}sniperfast_filters_position
            SET `is_open` = NOT `is_open`
            WHERE id_filter = " . (int) $id;

        return $this->db->execute($sql);
    }

    private function getEnabledFromDb($id, $type)
    {
        $sql = "SELECT `enabled`
        FROM {$this->dbp}sniperfast_" . pSQL($type) .
        ' WHERE id_' . pSQL($type) . ' = ' . (int) $id;

        return $this->db->getValue($sql);
    }

    public function getBaseUrl()
    {
        $baseUrl = Context::getContext()->link->getBaseLink();

        try {
            $sql = 'SELECT domain, virtual_uri
                FROM ' . _DB_PREFIX_ . 'shop_url
                WHERE id_shop = ' . (int) $this->idShop;
            $uris = Db::getInstance()->executeS($sql);

            $baseUrlClean = rtrim($baseUrl, '/');
            $baseUrlClean = str_replace(['http://', 'https://'], '', $baseUrlClean);

            foreach ($uris as $uri) {
                if ($uri['domain'] != $baseUrlClean) {
                    continue;
                }

                if (!empty($uri['virtual_uri'])) {
                    $baseUrl .= $uri['virtual_uri'];
                }
            }
        } catch (Exception $e) {
        }

        $baseUrl = rtrim($baseUrl, '/');

        return $baseUrl;
    }

    public function writeCsvRow($data, $filePointer)
    {
        $csvRow = [
            $data['id_product'],
            $data['id_category_default'],
            $data['reference'],
            $data['ean13'],
            $data['pname'],
            $data['description'],
            $data['description_short'],
            $data['supplier_name'],
            $data['category_link_rewrite'],
            $data['plink'],
            $data['mname'],
            $data['quantity'],
            $data['supplier_reference'],
            $data['cover_image_url'],
            $data['psales'],
            $data['category'],
            $data['category/lvl1'],
            $data['category/lvl2'],
            $data['category/lvl3'],
            $data['category/lvl4'],
            $data['price_deleted'],
            $data['price_showed'],
            $data['features'],
            $data['attributes'],
            $data['tags'],
            $data['alternate_image_url'],
        ];

        return fputcsv($filePointer, $csvRow, '|', '"');
    }

    public function getEnabledProducts(SniperfastProductFilter $filter)
    {
        $cacheDir = dirname(__FILE__) . '/../tmp/';
        $cacheFile = "{$this->idShop}_{$this->idLang}_enabled_products.json";

        if (file_exists($cacheDir . $cacheFile)) {
            return json_decode(Tools::file_get_contents($cacheDir . $cacheFile), true);
        }

        $out = null;
        $out = $filter->getEnabledProducts();

        if (!file_exists($cacheDir)) {
            $old_mask = umask();
            umask(0);
            mkdir($cacheDir, 0775);
            umask($old_mask);
        }

        file_put_contents($cacheDir . $cacheFile, json_encode($out));

        return $out;
    }

    public function cleanMemory()
    {
        static $last_memory = false;

        if ($last_memory === false) {
            $last_memory = memory_get_usage();
        }

        $current_memory = memory_get_usage();
        if ($current_memory - $last_memory < (32 * 1048576)) {
            return false;
        }

        Cache::clean('*');
        Product::flushPriceCache();
        $last_memory = $current_memory;

        return true;
    }

    public function sendFetchRequest($token, &$idJob, $moduleVersion)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, CDN_PROTOCOL . '://index.sniperfast.' . CDN_ENDPOINT . '/schedule');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query([
            'shop' => $this->idShop,
            'lang' => $this->idLang,
            'key' => SniperCommon::getConfig('sniper_key', $this->idShop, $this->idLang),
            'index' => SniperCommon::getConfig('sniper_index_name', $this->idShop, $this->idLang),
            'token' => $token,
            'ip' => $_SERVER['SERVER_ADDR'],
            'v' => $moduleVersion,
        ]));

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $res = curl_exec($ch);
        curl_close($ch);
        $res = json_decode($res, true);

        if (isset($res['id_job'])) {
            $idJob = $res['id_job'];
        }

        if (isset($res['status']) && $res['status'] == 'ok') {
            return 1;
        }

        return 0;
    }

    public function checkExpirationDate($isFromBo, &$result, &$msg)
    {
        $baseUrl = _PS_BASE_URL_ . __PS_BASE_URI__;
        if (Configuration::get('PS_SSL_ENABLED')) {
            $baseUrl = str_replace('http://', 'https://', $baseUrl);
        }
        $baseUrl = rtrim($baseUrl, '/');

        $indexName = SniperCommon::getConfig('sniper_index_name', $this->idShop, $this->idLang);
        $indexKey = SniperCommon::getConfig('sniper_key', $this->idShop, $this->idLang);
        $indexes = [];
        $sql = "SELECT conf_value
            FROM {$this->dbp}sniperfast_conf
            WHERE conf_key = 'sniper_index_name'
            AND id_shop = " . (int) $this->idShop;
        $indexesRes = $this->db->executeS($sql);
        if (!empty($indexesRes)) {
            foreach ($indexesRes as $ir) {
                $indexes[] = $ir['conf_value'];
            }
        }

        if (empty($indexes)) {
            $msg = 'invalid_indexes';

            return false;
        }

        $params = [
            'index_name' => $indexName,
            'index_key' => $indexKey,
            'indexes' => $indexes,
            'iso_code' => Context::getContext()->language->iso_code,
            'from_bo' => $isFromBo,
        ];

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, CDN_PROTOCOL . '://index.sniperfast.' . CDN_ENDPOINT . '/getexpirationdate');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, 15);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $res = curl_exec($ch);
        curl_close($ch);
        $res = json_decode($res, true);

        if (empty($res)) {
            $msg = 'get_expiration_error';

            return false;
        }
        if (!$res['success']) {
            $msg = $res['msg'];

            return false;
        }

        $expirationDate = $res['expiration'];
        $expirationDateDt = new DateTime($expirationDate);
        $nowDt = new DateTime('now');
        $isExpired = false;

        if ($nowDt > $expirationDateDt) {
            $isExpired = true;
        }

        $expTime = strtotime($res['expiration']);
        $currentTimestamp = (int) microtime(true);
        SniperCommon::setConfig('expiration_date', $expTime, $this->idShop, 200000);
        $expirationFormatted = '';
        $days = SniperCommon::getDaysFromExpiration($expTime, $currentTimestamp, $expirationFormatted);

        $lastExpirationCheck = (int) SniperCommon::getConfig('last_expiration_check', $this->idShop, 200000);
        $checkRequired = false;
        if (empty($lastExpirationCheck)) {
            $checkRequired = true;
        } else {
            $checkRequired = SniperCommon::expirationCheckRequired(
                $days,
                $lastExpirationCheck,
                $currentTimestamp,
                $isExpired
            );
        }
        if ($checkRequired && !$isFromBo) {
            SniperCommon::setConfig('last_expiration_check', $currentTimestamp, $this->idShop, 200000);
        }

        if (!$isExpired) {
            if ($days < -14) {
                $result = [
                    'show_warning' => false,
                    'has_expired' => false,
                    'expiration_days' => abs($days),
                    'expiration_date' => $expirationFormatted,
                    'custom_message' => $res['custom_message'],
                ];
            } elseif ($days >= -14 && $days <= 0) {
                $result = [
                    'show_warning' => true,
                    'has_expired' => false,
                    'expiration_days' => abs($days),
                    'expiration_date' => $expirationFormatted,
                    'custom_message' => $res['custom_message'],
                ];
            }
        } else {
            $result = [
                'show_warning' => true,
                'has_expired' => true,
                'expiration_days' => abs($days),
                'expiration_date' => $expirationFormatted,
                'custom_message' => $res['custom_message'],
            ];
        }

        $msg = 'expiration_success';

        return true;
    }

    public function getAdminEmployee()
    {
        $sql = "SELECT MIN(e.id_employee) AS id_employee
            FROM {$this->dbp}employee e
            LEFT JOIN {$this->dbp}employee_shop es
                ON e.id_employee = es.id_employee
            LEFT JOIN {$this->dbp}profile_lang pl
                ON pl.id_profile = e.id_profile
            WHERE es.id_shop = " . (int) $this->idShop .
            ' AND pl.id_lang = ' . (int) $this->idLang .
            ' AND e.active = 1
            AND pl.name LIKE \'%Admin\'';
        $idEmployee = (int) $this->db->getValue($sql);
        if (!empty($idEmployee)) {
            return $idEmployee;
        }

        $sql = "SELECT MIN(e.id_employee) AS id_employee
            FROM {$this->dbp}employee e
            LEFT JOIN {$this->dbp}employee_shop es
                ON e.id_employee = es.id_employee
            WHERE es.id_shop = " . (int) $this->idShop .
            ' AND e.active = 1';

        return (int) $this->db->getValue($sql);
    }

    public function getSliderData($idCategory, $idManufacturer, $idProduct)
    {
        $sliderData = [
            'id' => 0,
            'category' => [
                'lv1' => '',
                'lv2' => '',
                'lv3' => '',
                'lv4' => '',
                'lv5' => '',
            ],
            'manufacturer' => '',
            'product_name' => '',
        ];

        if (!empty($idProduct)) {
            $product = new Product($idProduct, false, $this->idLang);
            $idCategory = $product->id_category_default;
            $idManufacturer = $product->id_manufacturer;
            $sliderData['product_name'] = $product->name;
        }
        if (!empty($idManufacturer)) {
            $sql = "SELECT `name`
                FROM {$this->dbp}manufacturer
                WHERE id_manufacturer = " . (int) $idManufacturer;
            $sliderData['manufacturer'] = $this->db->getValue($sql);
        }
        if (!empty($idCategory)) {
            $explodedPath = [];
            $path = Tools::getPath('', (int) $idCategory);
            $path = strip_tags($path);
            $path = str_replace(Configuration::get('PS_NAVIGATION_PIPE'), ' > ', $path);
            $explodedPath = explode(' > ', $path);
            if (count($explodedPath) > 1) {
                $explodedPath = array_slice($explodedPath, 1);
            }
            $explodedPath = array_filter(array_map('trim', $explodedPath));
            if (!empty($explodedPath)) {
                $catIndex = 1;
                foreach ($explodedPath as $e) {
                    $sliderData['category']['lv' . $catIndex] = $e;
                    ++$catIndex;
                }
            }
        }

        return $sliderData;
    }

    public function updateMainDataFromManager($settings, $bannerDesktop, $bannerMobile, $remarketingToken)
    {
        $sourceFile = _PS_MODULE_DIR_ . 'sniperfast/views/js/sniperfast_data_source.js';
        $template = Tools::file_get_contents($sourceFile);
        if ($template === false) {
            $errorMsg = "Can\'t open template file";

            return false;
        }

        $indexName = SniperCommon::getConfig('sniper_index_name', $this->idShop, $this->idLang);
        if (empty($indexName)) {
            return false;
        }

        // Available filter
        $availableFilter = (int) (isset($settings['settings']['sf_noavail_filter']) ? $settings['settings']['sf_noavail_filter'] : 0);
        SniperCommon::setConfig('available_filter', $availableFilter, $this->idShop, $this->idLang);
        $configuredFeatures = SniperCommon::getConfig('sniper_features', $this->idShop, $this->idLang);
        if (empty($configuredFeatures)) {
            $configuredFeatures = [];
        } else {
            $configuredFeatures = explode(',', $configuredFeatures);
        }
        if ($availableFilter) {
            $configuredFeatures[] = 9950;
            $configuredFeatures = array_unique($configuredFeatures);
            $configuredFeatures = implode(',', $configuredFeatures);
        } elseif (in_array(9950, $configuredFeatures)) {
            $newConfiguredFeatures = [];
            foreach ($configuredFeatures as $cf) {
                if ($cf != 9950) {
                    $newConfiguredFeatures[] = $cf;
                }
            }
            $configuredFeatures = implode(',', $newConfiguredFeatures);
        } else {
            $configuredFeatures = implode(',', $configuredFeatures);
        }
        SniperCommon::setConfig('sniper_features', $configuredFeatures, $this->idShop, $this->idLang);

        // Features
        $featureRangeFile = _PS_MODULE_DIR_ . 'sniperfast/output/' . $this->idShop . '_' . $this->idLang . '_range_feature.json';
        $featureRange = '';
        if (file_exists($featureRangeFile)) {
            $featureRangeFileEncoded = Tools::file_get_contents($featureRangeFile);
            $featureRangeFileDecoded = json_decode($featureRangeFileEncoded, true);
            if (!empty($featureRangeFileDecoded) && count($featureRangeFileDecoded) > 0) {
                $tempFeatures = [];
                foreach ($featureRangeFileDecoded as $f) {
                    if ($f['is_range'] == 1) {
                        $tempFeatures[$f['name']] = 1;
                    }
                }
                if (count($tempFeatures) > 0) {
                    $featureRange = json_encode($tempFeatures);

                    $featureRange = str_replace("'", "\'", $featureRange);
                }
            }
        }

        // Dedicated page
        $dedicatedPageUrl = SniperCommon::getDedicatedPageUrl($this->idShop, $this->idLang);
        $idCurrency = SniperCommon::getConfig('sniper_currency', $this->idShop, $this->idLang);
        $sql = 'SELECT iso_code FROM ' . _DB_PREFIX_ . 'currency
            WHERE id_currency = ' . (int) $idCurrency;
        $currencyIsoCode = $this->db->getValue($sql);

        // Filters order
        $filtersPosition = [];
        if (isset($settings['filters']) && !empty($settings['filters'])) {
            $filtersPosition = $this->convertFiltersPosition($settings['filters']);
        }

        // Relevance order
        $relevancePosition = [];
        if (isset($settings['relevance']) && !empty($settings['relevance'])) {
            foreach ($settings['relevance'] as $rel) {
                $relevancePosition[] = $rel;
            }
        }

        $megashopCompatibility = SniperCommon::getConfig('megashop_compatibility', $this->idShop, $this->idLang);
        $pageWidth = 992;
        $sniperResultWidthBody = 993;

        if ($megashopCompatibility) {
            $pageWidth = 1200;
            $sniperResultWidthBody = 1201;
        }

        $baseUrl = _PS_BASE_URL_ . __PS_BASE_URI__;
        if (Configuration::get('PS_SSL_ENABLED')) {
            $baseUrl = str_replace('http://', 'https://', $baseUrl);
        }

        // Banner
        $desktopBanner = '';
        $mobileBanner = '';
        $this->cleanPreviousBanners();
        if (!empty($bannerDesktop)) {
            if (!isset($bannerDesktop['ext'])) {
                $desktopBanner = $this->moveBanner('d', $bannerDesktop);
            } else {
                $desktopBanner = $this->getBanner('d', $bannerDesktop);
            }

            if (!empty($desktopBanner)) {
                $desktopBanner = $baseUrl . '/modules/sniperfast/resources/searchbanner/' . $desktopBanner;
            }
        }
        if (!empty($bannerMobile)) {
            if (!isset($bannerMobile['ext'])) {
                $mobileBanner = $this->moveBanner('m', $bannerMobile);
            } else {
                $mobileBanner = $this->getBanner('m', $bannerMobile);
            }

            if (!empty($mobileBanner)) {
                $mobileBanner = $baseUrl . '/modules/sniperfast/resources/searchbanner/' . $mobileBanner;
            }
        }

        $identity = $this->idShop . '_' . $this->idLang;

        // CSS
        if (isset($settings['custom_css'])) {
            if (empty($settings['custom_css']) && file_exists(_PS_MODULE_DIR_ . 'sniperfast/views/css/' . $identity . '_custom.css')) {
                @unlink(_PS_MODULE_DIR_ . 'sniperfast/views/css/' . $identity . '_custom.css');
            } elseif (!empty($settings['custom_css'])) {
                file_put_contents(_PS_MODULE_DIR_ . 'sniperfast/views/css/' . $identity . '_custom.css', $settings['custom_css']);
            }
        }

        // Synonyms
        if (isset($settings['synonyms'])) {
            if (empty($settings['synonyms']) && file_exists(_PS_MODULE_DIR_ . 'sniperfast/output/' . $identity . '_synonims.txt')) {
                @unlink(_PS_MODULE_DIR_ . 'sniperfast/output/' . $identity . '_synonims.txt');
            } elseif (!empty($settings['synonyms'])) {
                file_put_contents(_PS_MODULE_DIR_ . 'sniperfast/output/' . $identity . '_synonims.txt', $settings['synonyms']);
            }
        }

        // Logo
        if (!isset($settings['settings']['sf_website_logo']) && empty($settings['settings']['sf_website_logo'])) {
            $context = Context::getContext();
            $website_logo = Configuration::get('PS_LOGO', $context->shop->id, null, $context->language->id);
            $logoUrl = _PS_BASE_URL_ . __PS_BASE_URI__ . 'img/' . $website_logo;
        } else {
            $logoUrl = $settings['settings']['sf_website_logo'];
        }

        // Custom JS/CSS
        if (isset($settings['settings']['custom_link_js'])) {
            SniperCommon::setConfig('custom_link_js', $settings['settings']['custom_link_js'], $this->idShop, $this->idLang);
        }

        if (isset($settings['settings']['custom_link_css'])) {
            SniperCommon::setConfig('custom_link_css', $settings['settings']['custom_link_css'], $this->idShop, $this->idLang);
        }

        // Dynamic settings
        $dynamicSettings = [];
        foreach ($settings['settings'] as $dsKey => $dsValue) {
            if (Tools::strpos($dsKey, 'var_') === 0) {
                $dynamicSettings[str_replace('var_', '', $dsKey)] = $dsValue;
            }
        }

        $templateType = 'wide';
        $resultsWidth = '100';
        if (isset($settings['settings']['sf_template']) && !empty($settings['settings']['sf_template'])) {
            $templateType = $settings['settings']['sf_template'];
        }
        if (isset($settings['settings']['sf_res_width']) && !empty($settings['settings']['sf_res_width'])) {
            $resultsWidth = $settings['settings']['sf_res_width'];
        }

        $submittedValues = [
            '[[sniper_div_dsk]]' => isset($settings['settings']['sf_div_desktop']) ? $settings['settings']['sf_div_desktop'] : '',
            '[[sniper_div_mob]]' => isset($settings['settings']['sf_div_mobile']) ? $settings['settings']['sf_div_mobile'] : '',
            '[[sniper_index_name]]' => $indexName,
            '[[sniper_key]]' => SniperCommon::getConfig('sniper_key', $this->idShop, $this->idLang),
            '[[sniper_export_review]]' => SniperCommon::getConfig('sniper_export_review', $this->idShop, $this->idLang),
            '[[sniper_page_width]]' => $pageWidth,
            '[[warehouse_compatibility]]' => SniperCommon::getConfig('warehouse_compatibility', $this->idShop, $this->idLang),
            '[[megashop_compatibility]]' => $megashopCompatibility,
            '[[sniperfast_lang_iso]]' => Language::getIsoById($this->idLang),
            '[[sniperfast_dedicate_page]]' => $dedicatedPageUrl,
            '[[sniperfast_feature_range]]' => $featureRange,
            '[[sniper_result_width]]' => $resultsWidth,
            '[[sniper_result_width_body]]' => $sniperResultWidthBody,
            '[[sniper_result_align]]' => isset($settings['settings']['sf_res_align']) ? $settings['settings']['sf_res_align'] : '',
            '[[sniper_price_token]]' => SniperCommon::getConfig('price_token', $this->idShop, $this->idLang),
            '[[sniper_currency_iso]]' => $currencyIsoCode,
            '[[sniper_lens_redirect]]' => SniperCommon::getConfig('lens_redirect_enable', $this->idShop, $this->idLang),
            '[[sniper_filters_position]]' => json_encode($filtersPosition),
            '[[sniperfast_relevance_position]]' => json_encode($relevancePosition),
            '[[sniper_enable_search]]' => isset($settings['settings']['sf_enable']) ? $settings['settings']['sf_enable'] : 0,
            '[[sniper_result_height]]' => isset($settings['settings']['sf_result_height']) ? $settings['settings']['sf_result_height'] : '',
            '[[sniper_alternative_q]]' => (int) isset($settings['settings']['sf_alternative_q']) ? $settings['settings']['sf_alternative_q'] : '',
            '[[sniper_template_type]]' => $templateType,
            '[[sniper_hide_filters]]' => isset($settings['settings']['sf_hide_side_filters']) ? (int) $settings['settings']['sf_hide_side_filters'] : 0,
            '[[sniper_hide_prices]]' => isset($settings['settings']['hide_price']) ? (int) $settings['settings']['hide_price'] : 0,
            '[[sniper_realtime_prices]]' => isset($settings['settings']['realtime_price']) ? (int) $settings['settings']['realtime_price'] : 0,
            '[[sniper_remarketing_token]]' => $remarketingToken,
            '[[search_banner_desktop]]' => $desktopBanner,
            '[[search_banner_mobile]]' => $mobileBanner,
            '[[search_banner_link]]' => isset($settings['settings']['sf_search_banner']) ? $settings['settings']['sf_search_banner'] : '',
            '[[search_banner_alt]]' => isset($settings['settings']['sf_search_banner_alt']) ? $settings['settings']['sf_search_banner_alt'] : '',
            '[[search_banner_target]]' => isset($settings['settings']['sf_banner_targetblank']) ? $settings['settings']['sf_banner_targetblank'] : 0,
            '[[search_website_logo]]' => $logoUrl,
            '[[dynamic_settings]]' => json_encode($dynamicSettings),
        ];

        $template = str_replace(
            array_keys($submittedValues),
            $submittedValues,
            $template
        );

        $date = new DateTime();
        $dateFormat = $date->format('Y-m-d_H-i-s');
        $newTemplateName = md5($dateFormat . '_' . $this->idShop . '_' . $this->idLang);
        $newTemplate = dirname(__FILE__) . '/../views/js/' . $this->idShop . '_' . $this->idLang . '_' . $newTemplateName . '.js';
        if (!file_put_contents($newTemplate, $template)) {
            $errorMsg = "Can\'t write template file";

            return false;
        } else {
            $oldFile = SniperCommon::getConfig('source_data_file', $this->idShop, $this->idLang);
            $oldFilePath = dirname(__FILE__) . '/../views/js/' . $this->idShop . '_' . $this->idLang . '_' . $oldFile . '.js';
            if (file_exists($oldFilePath)) {
                unlink($oldFilePath);
            }
            SniperCommon::setConfig('source_data_file', $newTemplateName, $this->idShop, $this->idLang);
        }

        return true;
    }

    private function convertFiltersPosition($filters)
    {
        $customData = [];
        if (file_exists(_PS_MODULE_DIR_ . 'sniperfast/resources/' . $this->idShop . '_' . $this->idLang . '_customdata.json')) {
            $customData = json_decode(Tools::file_get_contents(_PS_MODULE_DIR_ . 'sniperfast/resources/' . $this->idShop . '_' . $this->idLang . '_customdata.json'), true);
        }

        foreach ($filters as &$f) {
            if ($f['t'] == 'pdata'
                || ($f['t'] == 'feat' && $f['v'] == 9950)
            ) {
                continue;
            }

            $sql = '';
            switch ($f['t']) {
                case 'attr':
                    $sql = "SELECT id_attribute_group
                        FROM {$this->dbp}attribute_group_lang";
                    break;
                case 'feat':
                    $sql = "SELECT id_feature
                        FROM {$this->dbp}feature_lang";
                    break;
            }
            $sql .= ' WHERE id_lang = ' . (int) $this->idLang .
            ' AND `name` = \'' . pSQL($f['v']) . '\'';

            $newValue = 0;
            try {
                $newValue = (int) $this->db->getValue($sql);
            } catch (Exception $e) {
            }

            if (!empty($newValue)) {
                $f['v'] = $newValue;
            } elseif (isset($customData[$f['v']])) {
                $f['v'] = $customData[$f['v']]['id'];
            }
        }
        unset($f);

        return $filters;
    }

    private function cleanPreviousBanners()
    {
        $folderPath = _PS_MODULE_DIR_ . 'sniperfast/resources/searchbanner/';
        $previousBanner = glob($folderPath . 'search_banner_*.*');
        if (empty($previousBanner)) {
            return;
        }

        foreach ($previousBanner as $pb) {
            try {
                @unlink($pb);
            } catch (Exception $e) {
            }
        }
    }

    private function moveBanner($bType, $banner)
    {
        $type = '';
        switch ($bType) {
            case 'd':
                $type = 'desktop';
                break;
            case 'm':
                $type = 'mobile';
                break;
        }
        $date = new DateTime();
        $dateFormat = $date->format('Y-m-d_H-i-s');
        $random = md5($type . '_' . $this->idShop . '_' . $this->idLang . '_' . $dateFormat);
        $fileName = explode('.', $banner['name']);
        $bannerName = 'search_banner_' . $type . '_' . $random . '.' . $fileName[1];

        if (!move_uploaded_file($banner['tmp_name'], _PS_MODULE_DIR_ . 'sniperfast/resources/searchbanner/' . $bannerName)) {
            return '';
        }

        return $bannerName;
    }

    public function getBanner($bType, $banner)
    {
        $type = '';
        switch ($bType) {
            case 'd':
                $type = 'desktop';
                break;
            case 'm':
                $type = 'mobile';
                break;
        }
        $date = new DateTime();
        $dateFormat = $date->format('Y-m-d_H-i-s');
        $random = md5($type . '_' . $this->idShop . '_' . $this->idLang . '_' . $dateFormat);

        $bannerData = base64_decode($banner['img']); // Get image data
        $bannerName = 'search_banner_' . $type . '_' . $random . '.' . $banner['ext'];

        file_put_contents(_PS_MODULE_DIR_ . 'sniperfast/resources/searchbanner/' . $bannerName, $bannerData);

        if ($bannerData === false) {
            return '';
        }

        return $bannerName;
    }

    public function sendConversionDataToManager($isOnboarding, $idJob, $conversionType, $selectedLangs = [])
    {
        $isoCodes = [];
        if (!empty($selectedLangs)) {
            foreach ($selectedLangs as $sl) {
                $language = new Language($sl, $sl);
                $isoCodes[] = $language->iso_code;
            }
        }
        $params = [
            'is_onboarding' => $isOnboarding,
            'index_name' => SniperCommon::getConfig('sniper_index_name', $this->idShop, $this->idLang),
            'type' => $conversionType,
            'id_job' => $idJob,
            'selected_langs' => implode(',', $isoCodes),
        ];

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, MANAGER_ENDPOINT . '/onboarding/setstatus');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        try {
            $res = curl_exec($ch);
            curl_close($ch);
            $res = json_decode($res, true);
            if (empty($res)) {
                $msg = 'conversion_error';

                return false;
            }
            if (!isset($res['success']) || !$res['success']) {
                $msg = 'unexpected_error';
                if (isset($res['msg'])) {
                    $msg = $res['msg'];
                }

                return false;
            }

            return true;
        } catch (Exception $e) {
            $msg = $e->getMessage();

            return false;
        }
    }

    public function sendIndexRequestToManager(&$msg)
    {
        $indexName = SniperCommon::getConfig('sniper_index_name', $this->idShop, $this->idLang);
        $token = SniperCommon::getConfig('migration_token', $this->idShop, 200000);

        if (!$indexName || !$token) {
            return false;
        }

        $params = [
            'index_name' => $indexName,
            'token' => $token,
        ];

        // Append file
        $moduleDir = _PS_MODULE_DIR_ . 'sniperfast/output/';
        $synonimsFile = $this->idShop . '_' . $this->idLang . '_synonims.txt';
        $exportFile = $this->idShop . '_' . $this->idLang . '_export.csv';
        $zipFile = $this->idShop . '_' . $this->idLang . '_export.zip';

        if (!file_exists($moduleDir . $exportFile)) {
            $msg = 'Feed not found';
            return false;
        }

        /* Prepare ZIP file */
        if (file_exists($moduleDir . $zipFile)) {
            unlink($moduleDir . $zipFile);
        }

        $zip = new ZipArchive();
        if ($zip->open($moduleDir . $zipFile, ZipArchive::CREATE) !== true) {
            $msg = 'Could not create zip.';
            return false;
        }
        // add the files
        $zip->addFile($moduleDir . $exportFile, $exportFile);
        if (file_exists($moduleDir . $synonimsFile)) {
            $zip->addFile($moduleDir . $synonimsFile, $synonimsFile);
        }

        $zip->close();

        $file = new CURLFile(realpath($moduleDir . $zipFile), 'application/zip');
        $params['file'] = $file;

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, MANAGER_ENDPOINT . '/sniperfast/cmsindex');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        try {
            $res = curl_exec($ch);
            curl_close($ch);
            $res = json_decode($res, true);
            if (empty($res)) {
                $msg = 'unexpected_error';
                return false;
            }

            if (!isset($res['success']) || !$res['success']) {
                $msg = 'unexpected_error';
                if (isset($res['msg'])) {
                    $msg = $res['msg'];
                }
            } else {
                $msg = $res['msg'];
            }
        } catch (Exception $e) {
            $msg = $e->getMessage();
        }

        return $res;
    }

    public function getSearchPathFromManager($selectedLangs)
    {
        $indexList = [];
        foreach ($selectedLangs as $sl) {
            $index = SniperCommon::getConfig('sniper_index_name', $this->idShop, $sl);
            if (!empty($index)) {
                $indexList[] = $index;
            }
        }
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, MANAGER_ENDPOINT . '/searchdetect/subscription?' . http_build_query([
            'url' => $this->getBaseUrl(),
            'idx' => implode(',', $indexList),
        ]));
        curl_setopt($ch, CURLOPT_TIMEOUT, 10);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $res = curl_exec($ch);

        if (curl_errno($ch)) {
            $errorMsg = curl_error($ch);
            curl_close($ch);

            return false;
        }

        curl_close($ch);

        if (empty($res)) {
            $errorMsg = 'connection_error';

            return false;
        }

        $responseData = json_decode($res, true);

        if (!isset($responseData['success']) || !$responseData['success']) {
            $errorMsg = 'invalid_data';

            return false;
        }

        return $responseData['path'];
    }

    public function handleSearchActivation($selectedLangs, $enableSearch, $path, &$errorMsg)
    {
        $indexes = [];
        foreach ($selectedLangs as $sl) {
            $indexes[] = SniperCommon::getConfig('sniper_index_name', $this->idShop, $sl);
        }

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, MANAGER_ENDPOINT . '/sniperfast/handlesearchactivation');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, 10);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query([
            'token' => SniperCommon::getConfig('migration_token', $this->idShop, 200000),
            'index_list' => $indexes,
            'enable_search' => $enableSearch,
            'path' => $path,
        ]));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $res = curl_exec($ch);

        if (curl_errno($ch)) {
            $errorMsg = curl_error($ch);
            curl_close($ch);

            return false;
        }

        curl_close($ch);

        if (empty($res)) {
            $errorMsg = 'connection_error';

            return false;
        }

        $responseData = json_decode($res, true);

        if (!isset($responseData['success']) || !$responseData['success']) {
            $errorMsg = 'invalid_data';

            return false;
        }

        return true;
    }
}
