<?php
/**
 *  Copyright (C) Prestalia - All Rights Reserved
 *  Unauthorized copying of this file, via any medium is strictly prohibited
 *  Proprietary and confidential
 *
 *  @author    Prestalia <prestalia.it>
 *  @copyright 2019 - 2024 Prestalia
 *  @license   Commercial
 */
if (!defined('_PS_VERSION_')) {
    exit;
}
header('Content-Type: application/json; charset=utf-8');
header('Access-Control-Allow-Origin: *');
header('Cache-Control: no-cache, must-revalidate');
header('Expires: Sat, 26 Jul 1997 05:00:00 GMT');

require_once dirname(__FILE__) . '/../../autoload.php';

use Prestalia\Sniperfast\AjaxValidator\AjaxValidator;

class AdminSniperfastSettingsController extends ModuleAdminController
{
    private $idShop;
    private $idLang;
    private $migrationToken;

    private $managerEmail;
    private $managerPassword;

    public function postProcess()
    {
        ob_start();

        $this->idShop = Tools::getValue('s');
        $this->idLang = Tools::getValue('l');

        $this->managerEmail = Tools::getValue('email', '');
        $this->managerEmail = trim($this->managerEmail);
        $this->managerPassword = Tools::getValue('password', '');
        $this->managerPassword = trim($this->managerPassword);

        $this->migrationToken = Tools::getValue('migration_token', '');
        $this->migrationToken = trim($this->migrationToken);

        $ajaxToken = Tools::getValue('t');

        $ajaxValidator = new AjaxValidator($this->idShop, $this->idLang, $ajaxToken);
        if (!$ajaxValidator->isValid()) {
            exit(json_encode($ajaxValidator->getError()));
        }

        parent::postProcess();
    }

    public function ajaxProcessRecoverAccount()
    {
        $sniperSettings = new SniperfastSettings($this->idShop, $this->idLang);
        $msg = '';

        if (empty($this->migrationToken)) {
            $migration_token = SniperfastSettings::getMigrationToken($this->managerEmail, $this->managerPassword);
            $this->migrationToken = $migration_token;
        }

        if (empty($this->migrationToken)) {
            $this->term(false, [], 'invalid_credentials');
        }

        $res = $sniperSettings->registerRecoverAccount($this->migrationToken, 'single', $msg);

        $config = [];
        $config[$this->idLang] = [
            'sniper_index_name' => SniperCommon::getConfig('sniper_index_name', $this->idShop, $this->idLang),
            'sniper_key' => SniperCommon::getConfig('sniper_key', $this->idShop, $this->idLang),
            'ajax_token' => SniperCommon::getConfig('ajax_token', $this->idShop, $this->idLang),
        ];
        $this->term($res, $config, $msg);
    }

    public function ajaxProcessRecoverAccountActive()
    {
        $config = [];
        $sql = 'SELECT conf_value, id_lang
            FROM ' . _DB_PREFIX_ . 'sniperfast_conf
            WHERE conf_key = \'sniper_index_name\'
            AND id_shop = ' . (int) $this->idShop;
        $configRes = Db::getInstance()->executeS($sql);

        if (empty($this->migrationToken)) {
            $migration_token = SniperfastSettings::getMigrationToken($this->managerEmail, $this->managerPassword);
            $this->migrationToken = $migration_token;
        }

        if (empty($this->migrationToken)) {
            $this->term(false, [], 'invalid_credentials');
        }

        $msgFull = [];
        $res = false;
        $conversionSent = false;
        $conversionData = [];
        if (!empty($configRes)) {
            foreach ($configRes as $c) {
                $msg = '';
                SniperCommon::checkConfig($this->idShop, $c['id_lang']);
                $sniperSettings = new SniperfastSettings($this->idShop, $c['id_lang']);
                $res = $sniperSettings->registerRecoverAccount($this->migrationToken, 'active', $msg);
                $idJob = SniperCommon::getConfig('last_job_id', $this->idShop, $c['id_lang']);

                if (!$conversionSent && !empty($idJob)) {
                    $sniperSettings->sendConversionDataToManager(1, $idJob, 2);
                    $conversionData = [
                        'name' => $c['conf_value'],
                        'id_job' => $idJob,
                    ];
                    $conversionSent = true;
                }

                $msgFull[] = $msg;
            }
        }

        $this->term($res, $conversionData, implode(' ', $msgFull));
    }

    public function ajaxProcessRecoverAccountFull()
    {
        $msg = '';
        $res = false;

        $migration_token = SniperfastSettings::getMigrationToken($this->managerEmail, $this->managerPassword);
        if (!empty($migration_token)) {
            $this->migrationToken = $migration_token;
        }

        if (empty($this->migrationToken)) {
            $this->term(false, [], 'invalid_credentials');
        }

        foreach (Language::getLanguages(true, $this->idShop) as $lang) {
            SniperCommon::checkConfig($this->idShop, $lang['id_lang']);
            $sniperSettings = new SniperfastSettings($this->idShop, $lang['id_lang']);
            $res = $sniperSettings->registerRecoverAccount($this->migrationToken, 'full', $msg);
        }

        $config = [];
        $sql = 'SELECT id_lang, conf_key, conf_value
            FROM ' . _DB_PREFIX_ . 'sniperfast_conf
            WHERE conf_key IN (\'sniper_index_name\', \'sniper_key\', \'ajax_token\')
            AND id_shop = ' . (int) $this->idShop;
        $configRes = Db::getInstance()->executeS($sql);
        if (!empty($configRes)) {
            foreach ($configRes as $c) {
                if (!isset($config[$c['id_lang']])) {
                    $config[$c['id_lang']] = [];
                }
                $config[$c['id_lang']][$c['conf_key']] = $c['conf_value'];
            }
        }

        $this->term($res, $config, $msg);
    }

    public function ajaxProcessEnableSearch()
    {
        $selectedLangs = Tools::getValue('selected_langs', []);
        $enableSearch = (int) Tools::getValue('enable_search', 0);

        if (empty($selectedLangs)) {
            $this->term(true, '', 'nothing_to_do');
        }
        $msg = '';
        $sniperSettings = new SniperfastSettings($this->idShop, $this->idLang);
        $searchPath = $sniperSettings->getSearchPathFromManager($selectedLangs);

        if (!$sniperSettings->handleSearchActivation($selectedLangs, $enableSearch, $searchPath, $msg)) {
            $this->term(false, '', $msg);
        }

        $res = [];
        foreach ($selectedLangs as $sl) {
            SniperCommon::updateMainData($this->idShop, $sl, $msg);
            $res[] = $sl . '-' . $msg;
            usleep(300000);
        }

        $this->term(true, '', implode(' ', $res));
    }

    public function ajaxProcessIndexRequestToManager()
    {
        $sniperSettings = new SniperfastSettings($this->idShop, $this->idLang);
        $msg = '';
        $res = $sniperSettings->sendIndexRequestToManager($msg);
        $this->term($res['success'], $res, $msg);
    }

    public function ajaxProcessFeatureRange()
    {
        $sniperSettings = new SniperfastSettings($this->idShop, $this->idLang);
        $msg = '';
        $range = Tools::getValue('val');
        $res = $sniperSettings->saveFeatureRange($range, $msg);
        $this->term($res, '', $msg);
    }

    public function ajaxProcessSaveJob()
    {
        $sniperSettings = new SniperfastSettings($this->idShop, $this->idLang);
        $msg = '';
        $idJob = Tools::getValue('id_job');
        $res = $sniperSettings->saveJobId($idJob, $msg);
        $isOnboarding = Tools::getValue('is_onboarding', 'false');
        if (is_string($isOnboarding)) {
            $isOnboarding = filter_var($isOnboarding, FILTER_VALIDATE_BOOLEAN);
        }
        $statusSent = Tools::getValue('status_sent', 'false');
        if (is_string($statusSent)) {
            $statusSent = filter_var($statusSent, FILTER_VALIDATE_BOOLEAN);
        }
        $conversionType = (int) Tools::getValue('conversion_type', 1);
        $selectedLangs = Tools::getValue('selected_langs', []);

        if ($isOnboarding && !$statusSent
            || !$isOnboarding
        ) {
            $sniperSettings->sendConversionDataToManager($isOnboarding, $idJob, $conversionType, $selectedLangs);
        }
        $this->term($res, '', $msg);
    }

    public function ajaxProcessConvertIndex()
    {
        $sniperSettings = new SniperfastSettings($this->idShop, $this->idLang);
        $msg = '';
        $res = $sniperSettings->convertIndex($msg);
        $this->term($res, '', $msg);
    }

    public function ajaxProcessDeleteIndexAndKey()
    {
        $sniperSettings = new SniperfastSettings($this->idShop, $this->idLang);
        $msg = '';
        $res = $sniperSettings->deleteIndexAndKey($msg);
        $this->term($res, '', $msg);
    }

    public function ajaxProcessFlipStatus()
    {
        $sniperSettings = new SniperfastSettings($this->idShop, $this->idLang);
        $msg = '';
        $id = (int) Tools::getValue('id');
        $type = Tools::getValue('type');
        $res = $sniperSettings->flipStatus($id, $type, $msg);
        $this->term($res, '', $msg);
    }

    public function ajaxProcessSelectorChange()
    {
        $lang = (int) Tools::getValue('lang');
        if (empty($lang) || $lang < 1) {
            $this->term(false, '', 'language_change_error');
        }

        SniperCommon::setConfig('selected_language', $lang, $this->idShop, 160000);
        $this->term(true, '', 'language_change_success');
    }

    public function ajaxProcessGetExpirationDate()
    {
        $sniperSettings = new SniperfastSettings($this->idShop, $this->idLang);
        $msg = '';
        $result = null;
        $isFromBo = (int) Tools::getValue('isFromBo', 0);
        $res = $sniperSettings->checkExpirationDate($isFromBo, $result, $msg);
        $this->term($res, $result, $msg);
    }

    public function ajaxProcessSaveBoWarning()
    {
        $isChecked = (int) Tools::getValue('checked', 0);
        SniperCommon::setConfig('hide_bo_warning', $isChecked, $this->idShop, 200000);
        $this->term(true, [], 'bowarn_save_success');
    }

    private function term($success, $result, $msg)
    {
        exit(json_encode([
            'success' => $success,
            'result' => $result,
            'msg' => $msg,
            'discarded_output' => ob_get_clean(),
        ]));
    }
}
