<?php
/**
 *  Copyright (C) Prestalia - All Rights Reserved
 *  Unauthorized copying of this file, via any medium is strictly prohibited
 *  Proprietary and confidential
 *
 *  @author    Prestalia <prestalia.it>
 *  @copyright 2019 - 2024 Prestalia
 *  @license   Commercial
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class SniperfastLandingPageModuleFrontController extends ModuleFrontController
{
    public function __construct()
    {
        require_once dirname(__FILE__) . '/../../autoload.php';

        parent::__construct();
        $this->context = Context::getContext();
    }

    public function initContent()
    {
        parent::initContent();

        $idLang = $this->context->language->id;
        $idShop = $this->context->shop->id;

        $baseUrl = _PS_BASE_URL_ . __PS_BASE_URI__;
        if (Configuration::get('PS_SSL_ENABLED')) {
            $baseUrl = str_replace('http://', 'https://', $baseUrl);
        }

        $idLanding = (int) Tools::getValue('id_landing');

        if (!is_numeric($idLanding)) {
            Tools::redirect('index.php?controller=404');
        }

        $landingPage = new SniperfastLandingPage($idShop, $idLang, $idLanding);
        $landingConfiguration = $landingPage->getLandingPageConfiguration();
        $categoryPath = $landingPage->getCategoriesPath($landingConfiguration);
        $sort = 'rel';
        if ($landingConfiguration['source'] == 1) {
            $sort = 'best';
        }

        $searchParameters = [
            'user_input' => $landingConfiguration['search_query'],
            'cat1' => $categoryPath['cat1'],
            'cat2' => $categoryPath['cat2'],
            'cat3' => $categoryPath['cat3'],
            'cat4' => $categoryPath['cat4'],
            'snmanu' => $landingPage->getManufacturerName($landingConfiguration),
            'sort' => $sort,
        ];

        $this->context->smarty->assign([
            'title' => $landingConfiguration['name'],
            'description' => $landingConfiguration['description'],
            'image' => $landingConfiguration['image'],
            'meta_title' => $landingConfiguration['meta_title'],
            'meta_description' => $landingConfiguration['meta_description'],
            'land_search_params' => json_encode($searchParameters),
        ]);

        $this->setTemplate('module:sniperfast/views/templates/front/landingPageResults.tpl');
    }
}
