<?php
/**
 *  Copyright (C) Prestalia - All Rights Reserved
 *  Unauthorized copying of this file, via any medium is strictly prohibited
 *  Proprietary and confidential
 *
 *  @author    Prestalia <prestalia.it>
 *  @copyright 2019 - 2024 Prestalia
 *  @license   Commercial
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

header('Content-Type: application/json; charset=utf-8');
header('Access-Control-Allow-Origin: *');
header('Cache-Control: no-cache, must-revalidate');
header('Expires: Sat, 26 Jul 1997 05:00:00 GMT');

require_once dirname(__FILE__) . '/../../autoload.php';

class SniperfastSniperDataModuleFrontController extends ModuleFrontController
{
    public function postProcess()
    {
        if (!Tools::getIsset('action')) {
            exit(json_encode([
                'success' => false,
                'msg' => 'missing_action',
            ]));
        }
        $this->processGetPrice();
        $this->processSaveConfiguration();

        exit(json_encode([
            'success' => false,
            'msg' => 'invalid_action',
        ]));
    }

    protected function processGetPrice()
    {
        if (Tools::getValue('action') != 'getPrice') {
            return;
        }

        // Check that the request comes from the same customer logged in at front office.
        $email = Tools::getValue('email');
        if (empty($email)) {
            exit(json_encode([
                'success' => false,
                'msg' => 'invalid_email',
            ]));
        } elseif (!$this->context->customer->logged) {
            exit(json_encode([
                'success' => false,
                'msg' => 'customer_nologin',
            ]));
        }
        $frontCustomer = new Customer();
        $frontCustomer = $frontCustomer->getByEmail($email);
        if (empty($frontCustomer)) {
            exit(json_encode([
                'success' => false,
                'msg' => 'invalid_customer',
            ]));
        }
        if ($frontCustomer->secure_key != $this->context->customer->secure_key) {
            exit(json_encode([
                'success' => false,
                'msg' => 'invalid_customer',
            ]));
        }

        $idShop = (int) $this->context->shop->id;
        $idLang = (int) $this->context->language->id;
        $st = Tools::getValue('st');
        if ($st != SniperCommon::getConfig('price_token', $idShop, $idLang)) {
            exit(json_encode([
                'success' => false,
                'msg' => 'price_error',
            ]));
        }

        $products = Tools::getValue('product_list');
        if (empty($products)) {
            exit(json_encode([
                'success' => false,
                'msg' => 'invalid_products',
            ]));
        }

        $productPrices = [];
        $currencyIsoCode = Tools::getValue('iso_code');
        $idCurrency = 0;

        if (empty($currencyIsoCode)) {
            $this->context->currency = Currency::getDefaultCurrency();
            $idCurrency = 1;
        } else {
            $sql = 'SELECT id_currency FROM ' . _DB_PREFIX_ . "currency
                WHERE iso_code = '" . $currencyIsoCode . "'";
            $idCurrency = Db::getInstance()->getValue($sql);
            if (!empty($idCurrency)) {
                $this->context->currency = new Currency($idCurrency);
            } else {
                $idCurrency = 1;
                $this->context->currency = Currency::getDefaultCurrency();
            }
        }

        $this->context->employee = new Employee(1, $idLang, $idShop);
        $priceDisplayMethod = Group::getPriceDisplayMethod($this->context->customer->id_default_group);
        $useTaxes = true;
        if ($priceDisplayMethod) {
            $useTaxes = false;
        }

        foreach ($products as $p) {
            $idProd = (int) $p['idprod'];
            $minMax = '';
            $productPrices[$idProd]['d'] =
                (float) round(Product::getPriceStatic($idProd, $useTaxes, null, 2, null, false, false, 1, false), 2);
            $productPrices[$idProd]['s'] =
                (float) round(
                    Product::getPriceStatic($idProd, $useTaxes, null, 2, null, false, true, 1, false, $this->context->customer->id),
                    2
                );

            if ($productPrices[$idProd]['d'] < $productPrices[$idProd]['s']) {
                $productPrices[$idProd]['d'] = $productPrices[$idProd]['s'];
            }

            $productPrices[$idProd]['d'] = number_format($productPrices[$idProd]['d'], 2, ',', '');
            $productPrices[$idProd]['s'] = number_format($productPrices[$idProd]['s'], 2, ',', '');
            $product = new Product($idProd, true, $idLang, $idShop);
            if (!empty($product->id) && $product->hasAttributes()) {
                $combinations = $this->getCombinations($product, $idLang, $useTaxes);
                $min = 999999;
                $max = -1;
                foreach ($combinations as $c) {
                    if ($c['price'] < $min) {
                        $min = $c['price'];
                    }
                    if ($c['price'] > $max) {
                        $max = $c['price'];
                    }
                }
                if ($min != $max) {
                    $min = number_format($min, 2, ',', '');
                    $max = number_format($max, 2, ',', '');
                    $minMax = $min . '|' . $max;
                }
            }
            $productPrices[$idProd]['mm'] = $minMax;
        }

        if (!empty($productPrices)) {
            exit(json_encode([
                'success' => true,
                'msg' => 'price_success',
                'result' => $productPrices,
            ]));
        } else {
            exit(json_encode([
                'success' => false,
                'msg' => 'price_error',
            ]));
        }
    }

    private function getCombinations($product, $idLang, $useTaxes)
    {
        if (!($product instanceof Product)) {
            throw new InvalidArgumentException('$product must be an instance of class Product');
        }
        if (!is_int($idLang) && $idLang >= 1) {
            throw new InvalidArgumentException('$idLang must be an integer greater than 1');
        }

        $attributes = $product->getAttributeCombinations($idLang);
        $combinations = [];
        foreach ($attributes as $attribute) {
            $combinations[$attribute['id_product_attribute']]['attributes'][] = $attribute;
        }
        foreach ($combinations as &$combination) {
            // Add combination price
            $combination['price'] =
                Product::getPriceStatic($product->id, $useTaxes, $combination['attributes'][0]['id_product_attribute'], 2);
        }

        return $combinations;
    }

    protected function processSaveConfiguration()
    {
        if (Tools::getValue('action') != 'updateSettings') {
            return;
        }

        $idShop = (int) Tools::getValue('s', 0);
        $idLang = (int) Tools::getValue('l', 0);
        $managerToken = Tools::getValue('manager_token');

        if ($managerToken != SniperCommon::getConfig('manager_token', $idShop, $idLang)) {
            exit(json_encode([
                'success' => false,
                'msg' => 'invalid_token',
            ]));
        }

        $savedSettings = Tools::getValue('manager_settings');
        if (empty($savedSettings)) {
            exit(json_encode([
                'success' => false,
                'msg' => 'invalid_settings',
            ]));
        }
        $bannerDesktop = Tools::fileAttachment('banner_desktop');
        $bannerMobile = Tools::fileAttachment('banner_mobile');
        $sniperSettings = new SniperfastSettings($idShop, $idLang);

        try {
            $savedSettings = json_decode($savedSettings, true);
            SniperCommon::setConfig('remarketing_token', $savedSettings['remarketing_token'], $idShop, $idLang);
            $sniperSettings->updateMainDataFromManager($savedSettings, $bannerDesktop, $bannerMobile, $savedSettings['remarketing_token']);
        } catch (Exception $e) {
            exit(json_encode([
                'success' => false,
                'msg' => 'settings_error',
                'err' => $e->getMessage(),
            ]));
        }

        exit(json_encode([
            'success' => true,
            'msg' => 'settings_updated',
        ]));
    }
}
