<?php
/**
 *  Copyright (C) Prestalia - All Rights Reserved
 *  Unauthorized copying of this file, via any medium is strictly prohibited
 *  Proprietary and confidential
 *
 *  @author    Prestalia <prestalia.it>
 *  @copyright 2015 - 2024 Prestalia
 *  @license   Commercial
 *
 *  @version   1.0.0
 */

namespace Prestalia\Sniperfast\ExclusiveLock\Locks;

if (!defined('_PS_VERSION_')) {
    exit;
}

use Prestalia\Sniperfast\ExclusiveLock\LockInterface;

class FlockLock implements LockInterface
{
    private $handle;
    private $lockfile_path;
    private $is_acquired = false;

    public function __construct($lockfile)
    {
        try {
            $this->handle = fopen($lockfile, 'w+');
        } catch (\Exception $e) {
            throw new \RuntimeException("Could not open lockfile '$lockfile'. Reason: " . $e->getMessage());
        }

        $this->lockfile_path = $lockfile;
    }

    public function __destruct()
    {
        $this->release();
        fclose($this->handle);
    }

    public function acquire($blocking = false)
    {
        if ($this->is_acquired) {
            return true;
        }

        $mask = LOCK_EX | LOCK_NB;
        if ($blocking) {
            $mask = LOCK_EX;
        }

        if (!flock($this->handle, $mask)) {
            return false;
        }

        $this->is_acquired = true;

        return true;
    }

    public function release()
    {
        if (!$this->is_acquired) {
            return true;
        }

        if (!flock($this->handle, LOCK_UN)) {
            throw new \RuntimeException("Could not release lock '$this->lockfile_path'");
        }

        $this->is_acquired = false;

        return true;
    }

    public function isAcquired()
    {
        return $this->is_acquired;
    }
}
