<?php
/**
 *  Copyright (C) Prestalia - All Rights Reserved
 *  Unauthorized copying of this file, via any medium is strictly prohibited
 *  Proprietary and confidential
 *
 *  @author    Prestalia <prestalia.it>
 *  @copyright 2019 - 2024 Prestalia
 *  @license   Commercial
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class Sniperfast extends Module
{
    public static $definition = [];
    protected $config_form = false;
    protected $idLang;

    public function __construct()
    {
        require_once dirname(__FILE__) . '/autoload.php';

        $this->name = 'sniperfast';
        $this->tab = 'front_office_features';
        $this->version = '2.0.2';
        $this->author = 'Prestalia';
        $this->module_key = '6528d1c630912f2a83b36dfe8f82b4b4';
        $this->need_instance = 0;
        $this->bootstrap = true;
        $this->controllers = [
            'AdminSniperfast',
        ];

        parent::__construct();

        $this->displayName = $this->l('SniperFast');
        $this->description = $this->l('The best search engine for your shop.');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall this module?');

        $this->ps_versions_compliancy = ['min' => '1.7', 'max' => _PS_VERSION_];
        $this->idLang = $this->context->language->id;
    }

    public function install()
    {
        SniperCommon::installTab(
            'AdminCatalog',
            'AdminSniperfast',
            $this->l('SniperFast'),
            $this->name
        );
        SniperCommon::installTab(
            null,
            'AdminSniperfastSettings',
            $this->l('SniperFast - Settings'),
            $this->name
        );

        require dirname(__FILE__) . '/installer/install.php';

        return parent::install()
            && $this->registerHook('displayHeader')
            && $this->registerHook('displayBackOfficeTop')
            && $this->registerHook('displayBackOfficeHeader')
            && $this->registerHook('moduleRoutes')
            && $this->registerHook('gSitemapAppendUrls');
    }

    public function uninstall()
    {
        SniperCommon::uninstallTab('AdminSniperfast');
        SniperCommon::uninstallTab('AdminSniperfastSettings');
        require dirname(__FILE__) . '/installer/uninstall.php';

        return parent::uninstall();
    }

    public function getContent()
    {
        // There's no configuration to show. Just redirect to the module controller
        Tools::redirectAdmin(
            'index.php?controller=AdminSniperfast&token='
            . Tools::getAdminTokenLite('AdminSniperfast')
        );
    }

    public function hookDisplayHeader()
    {
        $idShop = $this->context->shop->id;
        $idLang = $this->idLang;
        $sourceData = SniperCommon::getConfig('source_data_file', $idShop, $idLang);
        $cssPath = $this->_path . '/views/css/' . $idShop . '_app.css';

        // $isCheckout = $this->context->controller->php_self == 'order'
        //     || $this->context->controller->php_self == 'order-confirmation';
        $isApp = $this->context->cookie->in_app;

        // if ($isCheckout && $isApp) {
        if ($isApp) {
            $this->context->controller->registerJavascript(
                'sniperfast-app-js',
                'modules/' . $this->name . '/views/js/' . $idShop . '_sniperfast_app.js',
                ['position' => 'bottom', 'priority' => 150]
            );

            $this->context->controller->addCSS($cssPath);
        }

        if (empty($sourceData)) {
            return;
        }

        $sql = 'SELECT conf_key, conf_value
            FROM ' . _DB_PREFIX_ . 'sniperfast_conf
            WHERE id_shop = ' . (int) $idShop .
            ' AND id_lang = ' . (int) $idLang .
            ' AND conf_key IN (\'custom_link_js\', \'custom_link_css\')';
        $res = Db::getInstance()->executeS($sql);

        $cssLink = 'v2/200/sniperfast.css';
        $jsLink = 'v2/200/sniperfast.js';
        if (!empty($res)) {
            foreach ($res as $r) {
                if (empty($r['conf_value'])) {
                    continue;
                }
                if ($r['conf_key'] == 'custom_link_js') {
                    $jsLink = $r['conf_value'];
                } else {
                    $cssLink = $r['conf_value'];
                }
            }
        }

        $this->context->controller->addJQuery();
        $this->context->controller->addJQueryUI('ui.slider');
        $this->context->controller->registerJavascript(
            'modules-sniperfast-js',
            'modules/' . $this->name . '/views/js/' . $idShop . '_' . $idLang . '_' . $sourceData . '.js',
            ['position' => 'bottom', 'priority' => 150]
        );

        if (file_exists(dirname(__FILE__) . '/views/js/loader.js')) {
            $this->context->controller->registerJavascript(
                'sniperfast-loader-js',
                'modules/' . $this->name . '/views/js/loader.js',
                ['position' => 'bottom', 'priority' => 150]
            );
        }

        if (file_exists(dirname(__FILE__) . '/views/js/' . $idShop . '_' . $idLang . '_custom.js')) {
            $this->context->controller->registerJavascript(
                'modules-snipercustom-js',
                'modules/' . $this->name . '/views/js/' . $idShop . '_' . $idLang . '_custom.js',
                ['position' => 'bottom', 'priority' => 150]
            );
        }

        $this->context->controller->registerJavascript(
            'sniperfast-jqtouch-js',
            'https://cdnjs.cloudflare.com/ajax/libs/jqueryui-touch-punch/0.2.3/jquery.ui.touch-punch.min.js',
            ['server' => 'remote', 'position' => 'bottom', 'priority' => 150]
        );

        if (file_exists(dirname(__FILE__) . '/views/css/' . $idShop . '_' . $idLang . '_custom.css')) {
            $this->context->controller->addCSS($this->_path . 'views/css/' . $idShop . '_' . $idLang . '_custom.css');
        }

        $idCustomer = (int) $this->context->customer->id;
        $idDefaultGroup = (int) $this->context->customer->id_default_group;
        $customer_data = $idCustomer . '_' . $idDefaultGroup;

        $outputTpl =
            $this->context->smarty->createTemplate(dirname(__FILE__) . '/views/templates/front/customer_data.tpl');
        $outputTpl->assign([
            'customer_data' => $customer_data,
        ]);

        $sliderData = [];
        $remarketingToken = SniperCommon::getConfig('remarketing_token', $idShop, $idLang);

        if (!empty($remarketingToken)) {
            $sniperfastSettings = new SniperfastSettings($idShop, $idLang);
            $idPage = 0;
            $idCategory = 0;
            $idManufacturer = 0;
            $idProduct = 0;
            $idEmployee = $sniperfastSettings->getAdminEmployee();
            $this->context->employee = new Employee($idEmployee, $idLang, $idShop);
            switch ($this->context->controller->php_self) {
                case 'category':
                    $idPage = $idCategory = (int) Tools::getValue('id_category');
                    break;
                case 'manufacturer':
                    $idPage = $idManufacturer = (int) Tools::getValue('id_manufacturer');
                    break;
                case 'product':
                    $idPage = $idProduct = (int) Tools::getValue('id_product');
                    break;
            }
            $sliderData = $sniperfastSettings->getSliderData($idCategory, $idManufacturer, $idProduct);
            $sliderData['id'] = $idPage;
        }

        $forceActivationForDebug = false;
        if (Tools::getIsset('sniperfast_action') && Tools::getValue('sniperfast_action') == 'try_activate') {
            $forceActivationForDebug = true;
        }

        $customResources = [
            'js' => SniperCommon::getConfig('custom_link_js', $idShop, $idLang),
            'css' => SniperCommon::getConfig('custom_link_css', $idShop, $idLang),
        ];

        Media::addJsDef([
            'prestalia_sniperfast' => [
                'custom_resources' => $customResources,
                'controller' => [
                    'data' => $this->context->link->getModuleLink('sniperfast', 'SniperData', []),
                ],
                'slider_data' => $sliderData,
                'action' => [
                    'try_act' => $forceActivationForDebug,
                ],
            ],
        ]);

        return $outputTpl->fetch();
    }

    public function hookModuleRoutes()
    {
        $route = [
            'module-sniperfast' => [
                'controller' => 'LandingPage',
                'rule' => 'sf_{id_landing}-{rewrite}',
                'keywords' => [
                    'id_landing' => ['regexp' => '[0-9]+', 'param' => 'id_landing'],
                    'rewrite' => ['regexp' => '[_a-zA-Z0-9-\pL]*', 'param' => 'lp_rewrite'],
                ],
                'params' => [
                    'fc' => 'module',
                    'module' => 'sniperfast',
                ],
            ],
        ];

        return $route;
    }

    public function hookgSitemapAppendUrls($params)
    {
        $language = $params['lang'];
        $idLang = $language['id_lang'];
        $idShop = $this->context->shop->id;
        $landingPages = SniperfastLandingPage::getAllLandingPageForLanguage($idShop, $idLang);
        $landingLink = new SniperLandingLink();
        $landingPageLinks = [];
        foreach ($landingPages as $lp) {
            $landingPageLinks[] = [
                'type' => 'landingpage',
                'page' => 'landingpage',
                'link' => $landingLink->getSniperLandingLink($idShop, $idLang, $lp['id_landing']),
                'image' => false,
            ];
        }

        $tagPageUrl = SniperCommon::getTagPageUrl($idShop, $idLang);
        if (!empty($tagPageUrl)) {
            $landingPageLinks[] = [
                'type' => 'landingpage',
                'page' => 'landingpage',
                'link' => $tagPageUrl,
                'image' => false,
            ];

            $dedicatedPageUrl = SniperCommon::getDedicatedPageUrl($idShop, $idLang);
            $searchList = SniperCommon::getMostSearchedTerms($idShop, $idLang);
            if (!empty($dedicatedPageUrl) && !empty($searchList)) {
                foreach ($searchList as $search) {
                    $landingPageLinks[] = [
                        'type' => 'landingpage',
                        'page' => 'landingpage',
                        'link' => $dedicatedPageUrl . '?q=' . $search,
                        'image' => false,
                    ];
                }
            }
        }

        return $landingPageLinks;
    }

    public function hookDisplayBackOfficeHeader()
    {
        Media::addJsDef([
            'prestalia_sniperfast' => [
                'admin_settings_link_save' => $this->context->link->getAdminLink('AdminSniperfastSettings'),
                'id_shop' => $this->context->shop->id,
                'id_lang' => $this->context->language->id,
                'ajax_token' => SniperCommon::getConfig('ajax_token', $this->context->shop->id, $this->context->language->id),
            ],
        ]);

        $this->context->controller->addCSS(dirname(__FILE__) . '/views/css/expiry.css');
        $this->context->controller->addJS(dirname(__FILE__) . '/views/js/expiry.js');
    }

    public function hookDisplayBackOfficeTop()
    {
        $sql = 'SELECT conf_key, conf_value
            FROM ' . _DB_PREFIX_ . 'sniperfast_conf
            WHERE id_shop = ' . (int) $this->context->shop->id .
            ' AND id_lang = 200000
            AND conf_key IN (\'last_expiration_check\', \'hide_bo_warning\', \'expiration_date\')';
        $res = Db::getInstance()->executeS($sql);

        $sniperConf = [
            'last_expiration_check' => 0,
            'hide_bo_warning' => 0,
            'expiration_date' => 0,
        ];

        if (!empty($res)) {
            foreach ($res as $r) {
                $sniperConf[$r['conf_key']] = (int) $r['conf_value'];
            }
        } else {
            $boTopTpl = $this->context->smarty->createTemplate(dirname(__FILE__) . '/views/templates/admin/backoffice_top.tpl');
            $boTopTpl->assign([
                'adminLinkSniper' => $this->context->link->getAdminLink('AdminSniperfast'),
                'checkRequired' => true,
            ]);

            return $boTopTpl->fetch();
        }

        // If subscription expired, return
        $expiryDate = DateTime::createFromFormat('U.u', number_format($sniperConf['expiration_date'], 6, '.', ''));
        $currDate = new DateTime('now');

        $isExpired = false;
        if (!empty($sniperConf['expiration_date']) && $currDate > $expiryDate) {
            $isExpired = true;
        }

        // If there still are more than 14 days before expiration, return
        $currentTimestamp = microtime(true);
        $expirationFormatted = '';
        $days = SniperCommon::getDaysFromExpiration($sniperConf['expiration_date'], $currentTimestamp, $expirationFormatted);
        $checkRequired = SniperCommon::expirationCheckRequired($days, $sniperConf['last_expiration_check'], $currentTimestamp, $isExpired);

        if ($checkRequired) {
            $sniperConf['hide_bo_warning'] = 0;
            SniperCommon::setConfig('hide_bo_warning', 0, $this->context->shop->id, 200000);
        }

        if (!$checkRequired && ($sniperConf['hide_bo_warning'] == 1 || $days < -14)) {
            return;
        }

        $boTopTpl = $this->context->smarty->createTemplate(dirname(__FILE__) . '/views/templates/admin/backoffice_top.tpl');
        $boTopTpl->assign([
            'adminLinkSniper' => $this->context->link->getAdminLink('AdminSniperfast'),
            'checkRequired' => $checkRequired,
        ]);

        return $boTopTpl->fetch();
    }
}
