<?php
/**
 *  Copyright (C) Prestalia - All Rights Reserved
 *  Unauthorized copying of this file, via any medium is strictly prohibited
 *  Proprietary and confidential
 *
 *  @author    Prestalia <prestalia.it>
 *  @copyright 2019 - 2024 Prestalia
 *  @license   Commercial
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once dirname(__FILE__) . '/../autoload.php';

function upgrade_module_1_5_1($module)
{
    // Remove old files
    $filesToRemove = [
        '/views/css/back_150.css',
        '/views/js/back_150.js',
    ];

    $modulePath = _PS_MODULE_DIR_ . 'sniperfast';
    foreach ($filesToRemove as $f) {
        if (file_exists($modulePath . $f)) {
            unlink($modulePath . $f);
        }
    }

    // Create new database tables
    $db = Db::getInstance();
    $p = _DB_PREFIX_;
    $sql = [
        "CREATE TABLE IF NOT EXISTS `{$p}sniperfast_landing` (
        	`id_landing` INT(11) NOT NULL DEFAULT '1',
        	`id_shop` INT(11) NOT NULL DEFAULT '1',
        	`id_lang` INT(11) NOT NULL DEFAULT '1',
        	`name` VARCHAR(128) NOT NULL COLLATE 'utf8_unicode_ci',
        	`link_rewrite` VARCHAR(128) NOT NULL COLLATE 'utf8_unicode_ci',
        	`description` TEXT NULL COLLATE 'utf8_unicode_ci',
        	`image` TEXT NULL COLLATE 'utf8_unicode_ci',
        	`meta_title` VARCHAR(128) NULL DEFAULT NULL COLLATE 'utf8_unicode_ci',
        	`meta_description` VARCHAR(512) NULL DEFAULT NULL COLLATE 'utf8_unicode_ci',
        	`source` INT(11) NOT NULL DEFAULT '0',
        	`source_data` VARCHAR(255) NULL DEFAULT NULL COLLATE 'utf8_unicode_ci',
        	`search_query` VARCHAR(128) NULL DEFAULT NULL COLLATE 'utf8_unicode_ci',
        	`active` INT(11) NOT NULL DEFAULT '1',
        	PRIMARY KEY (`id_shop`, `id_lang`, `id_landing`)
        ) COLLATE='utf8_unicode_ci' ENGINE=InnoDB",
    ];
    $res = true;
    try {
        foreach ($sql as $query) {
            $res &= $db->execute($query);
        }
    } catch (PrestashopDatabaseException $ex) {
        file_put_contents(dirname(__FILE__) . '/sniper151.txt', $ex->getMessage() . "\n", FILE_APPEND);
    }
    try {
        $module->registerHook('moduleRoutes');
    } catch (PrestashopDatabaseException $ex) {
        file_put_contents(dirname(__FILE__) . '/sniper151.txt', 'Hook route: ' . $ex->getMessage() . "\n", FILE_APPEND);
    }
    try {
        $module->registerHook('gSitemapAppendUrls');
    } catch (PrestashopDatabaseException $ex) {
        file_put_contents(dirname(__FILE__) . '/sniper151.txt', 'Hook sitemap: ' . $ex->getMessage() . "\n", FILE_APPEND);
    }

    return true;
}
