<?php
/**
 *  Copyright (C) Prestalia - All Rights Reserved
 *  Unauthorized copying of this file, via any medium is strictly prohibited
 *  Proprietary and confidential
 *
 *  @author    Prestalia <prestalia.it>
 *  @copyright 2019 - 2024 Prestalia
 *  @license   Commercial
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once dirname(__FILE__) . '/../autoload.php';

function upgrade_module_1_5_3($module)
{
    unset($module);
    $idShop = Context::getContext()->shop->id;
    if ($idShop < 1) {
        $idShop = 1;
    }

    $db = Db::getInstance();
    $p = _DB_PREFIX_;

    $sql = "SELECT DISTINCT(id_lang)
        FROM {$p}sniperfast_conf
        WHERE id_lang < 999";
    $configuredLanguages = $db->executeS($sql);

    // Update DB
    $sql = [
        "CREATE TABLE IF NOT EXISTS `{$p}sniperfast_relevance_position` (
            `id_relevance` INT(11) NOT NULL AUTO_INCREMENT,
        	`id_shop` INT(11) NOT NULL DEFAULT '1',
        	`id_lang` INT(11) NOT NULL DEFAULT '1',
            `type` VARCHAR(16) NOT NULL DEFAULT '',
        	`position` INT(11) NOT NULL,
        	`active` INT(11) NOT NULL DEFAULT '1',
        	PRIMARY KEY (`id_relevance`),
            UNIQUE INDEX `id_relevance` (`id_shop`, `id_lang`, `type`)
        ) COLLATE='utf8_unicode_ci' ENGINE=InnoDB",
    ];
    $res = true;
    try {
        foreach ($sql as $query) {
            $res &= $db->execute($query);
        }
    } catch (PrestashopDatabaseException $ex) {
        file_put_contents(dirname(__FILE__) . '/sniper153.txt', $ex->getMessage() . "\n", FILE_APPEND);
    }
    foreach ($configuredLanguages as $cl) {
        SniperCommon::initRelevancePosition($idShop, $cl['id_lang']);
    }

    // Update css file
    $modulePath = _PS_MODULE_DIR_ . 'sniperfast';
    $errorMsg = '';
    foreach ($configuredLanguages as $cl) {
        SniperCommon::updateMainData($idShop, $cl['id_lang'], $errorMsg);
        if (!empty($errorMsg)) {
            file_put_contents(dirname(__FILE__) . '/sniper153.txt', $errorMsg . "\n", FILE_APPEND);
        }

        $cssHeightFile = $modulePath . '/views/css/sniperfast.css';
        if (file_exists($cssHeightFile)) {
            $cssWidth = SniperCommon::getConfig('sniper_result_width', $idShop, $cl['id_lang']);
            if (empty($cssWidth)) {
                $cssWidth = 100;
            }

            $megashopCompatibility = SniperCommon::getConfig('megashop_compatibility', $idShop, $cl['id_lang']);
            $sniperResultWidthBody = 993;

            if ($megashopCompatibility) {
                $sniperResultWidthBody = 1201;
            }

            $cssHeightTemp = Tools::file_get_contents($cssHeightFile);
            $cssHeightTemp = str_replace('[[sniper_result_width_body]]', $sniperResultWidthBody, $cssHeightTemp);
            $cssHeightTemp = str_replace('[[sniper_result_width]]', $cssWidth, $cssHeightTemp);
            file_put_contents($modulePath . '/views/css/' . $idShop . '_' . $cl['id_lang'] . '_sniperfast.css', $cssHeightTemp);
        }
    }

    return true;
}
