<?php
/**
 *  Copyright (C) Prestalia - All Rights Reserved
 *  Unauthorized copying of this file, via any medium is strictly prohibited
 *  Proprietary and confidential
 *
 *  @author    Prestalia <prestalia.it>
 *  @copyright 2019 - 2024 Prestalia
 *  @license   Commercial
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once dirname(__FILE__) . '/../autoload.php';

function upgrade_module_2_0_0($module)
{
    if (Tools::version_compare(_PS_VERSION_, '1.7.0.0', '<')) {
        return false;
    }

    $module->registerHook('displayBackOfficeTop');
    $module->registerHook('displayBackOfficeHeader');
    $module->unregisterHook('actionOrderStatusPostUpdate');
    $module->unregisterHook('OrderConfirmation');
    $module->unregisterHook('displayAdminOrderTabShip');
    $module->unregisterHook('actionProductAdd');
    $module->unregisterHook('actionProductAttributeUpdate');
    $module->unregisterHook('actionProductUpdate');

    // Create new admin tab
    $tab = new Tab();
    $tab->id_parent = -1;
    $tab->name = [];

    foreach (Language::getLanguages(true) as $lang) {
        $tab->name[$lang['id_lang']] = $module->l('SniperFast - Settings');
    }

    $tab->class_name = 'AdminSniperfastSettings';
    $tab->module = 'sniperfast';
    $tab->active = 1;
    $tab->add();

    // Delete unused files
    $unusedFiles = [
        'sniperfast_16.php',
        'sniperfast_17.php',
        'ajax/doExportFunctions.php',
        'ajax/getPriceForApp.php',
        'ajax/saveAppData.php',
        'ajax/saveAppDataFunctions.php',
        'ajax/saveData.php',
        'ajax/saveDataFunctions.php',
        'ajax/setLanguage.php',
        'ajax/flipStatusProduct.php',
    ];
    foreach ($unusedFiles as $uf) {
        $filePath = _PS_MODULE_DIR_ . 'sniperfast/' . $uf;
        if (file_exists($filePath)) {
            try {
                @unlink($filePath);
            } catch (Exception $e) {
            }
        }
    }

    $appFilesFolders = ['app_api', 'app_api/handlers'];
    foreach ($appFilesFolders as $aff) {
        $phpFiles = glob(_PS_MODULE_DIR_ . 'sniperfast/' . $aff . '/*.php');
        if (empty($phpFiles)) {
            continue;
        }
        foreach ($phpFiles as $pf) {
            if (file_exists($pf)) {
                try {
                    @unlink($pf);
                } catch (Exception $e) {
                }
            }
        }
    }

    $db = Db::getInstance();
    $sql = 'ALTER TABLE ' . _DB_PREFIX_ . 'sniperfast_filters_position
        ADD COLUMN `has_search` TINYINT(1) UNSIGNED NOT NULL DEFAULT \'0\' AFTER `value`';
    $db->execute($sql);

    $sql = 'ALTER TABLE ' . _DB_PREFIX_ . 'sniperfast_filters_position
        ADD COLUMN `is_open` TINYINT(1) UNSIGNED NOT NULL DEFAULT \'0\' AFTER `has_search`';
    $db->execute($sql);

    return true;
}
