/**
 *  Copyright (C) Prestalia - All Rights Reserved
 *  Unauthorized copying of this file, via any medium is strictly prohibited
 *  Proprietary and confidential
 *
 *  @author    Prestalia <prestalia.it>
 *  @copyright 2019 - 2024 Prestalia
 *  @license   Commercial
 */
var sniper_exportInfo = {};
var statsClicked = false;
var sniperPanelWidthInit = false;
var selectedLangsOnboard = [];
var isOnboarding = false;
var statusRequestSent = false;
var conversionType = 1;

$(document).ready(function () {
    // Version check
    checkLatestVersion();
    checkSniperfastExpirationDateFromModule();
    $('#sniperfast-expiration-summary').on('change', 'input[name="hide_bowarn_enable"]', handleBoWarnChange);

    /* Account */
    $('#submit-account').click(registerRecoverAccount);
    $('#submit-account-full').click(registerRecoverAccountFull);
    $('#snpf-onboard-selall').on('change', selectDeselectAllLangs);
    $('#snpf-onboard-startexport').on('click', startMultipleExports);
    $('#snpf-onboard-startexport-contr').on('click', registerRecoverAccountStartExp);
    $('a[id^="snpf-open-manager"], a[id^="snpf-try-now"]').on('click', showGoToModuleButton);
    $('#snpf-go-to-module').on('click', showWarningGoToModule);
    $('input[name="sniperfast_enable"]').on('change', activateDeactivateSearch);

    if (typeof sniper_config_changed !== 'undefined' && sniper_config_changed == '1') {
        showConfigChangedMessage();
    }

    /* Ajax checkbox */
    $('.sniperfast-ajax-checkbox-lite').on('click', ajaxCheckboxLiteClick);

    // Language selector
    $('#lang-selector').change(languageSelectorChange);

    // Add click event to tabs
    $('[data-tab-sniperfast]').click(tabClick);
    // Open requested tab
    var idTab = $("div.sniperfast-container-tabs a.active").attr('data-tab-sniperfast');
    if (idTab === false) {
        idTab = 1;
    }
    selectTab('main', idTab);

    // Dedicated page
    $("input[type='radio'][name='dedi_page_enable']").change(dedicatedPageEnableChange);
    dedicatedPageEnableChange();
    $('#dedi_page_title').on('keyup', function(e){
        if (isArrowKey(e)) {
            return;
        }
        $("input[type='text'][name='dedi_page_rewrite']").val(
            sniperStr2url($("input[type='text'][name='dedi_page_title']").val(), 'UTF-8')
        );
    });
    // SEO page
    $('#tag_page_title').on('keyup', function(e){
        if (isArrowKey(e)) {
            return;
        }
        $("input[type='text'][name='tag_page_rewrite']").val(
            sniperStr2url($("input[type='text'][name='tag_page_title']").val(), 'UTF-8')
        );
    });
    // Landing page
    $('[id^="landing_title"]').on('keyup', function(e){
        if (isArrowKey(e)) {
            return;
        }
        var landingTitleNum = $(this).attr('id');
        landingTitleNum = landingTitleNum.replace('landing_title_', '');
        $("input[type='text'][name='landing_rewrite_"+landingTitleNum+"']").val(
            sniperStr2url($("input[type='text'][name='landing_title_"+landingTitleNum+"']").val(), 'UTF-8')
        );
    });

    $('#landing_source').change(showHideLandingSourceData);
    showHideLandingSourceData();
    // Fetch indexing status
    fetchIndexingStatus();

    /* Export btn */
    $('#btn-export').click(exportBtnClick);
    $('.snpf_show_advanced_cron').click(showAdvancedCron);

    /* Utilities */
    $('#sniperfast-container').on('click', '#delete-index-key', deleteIndexNameAndKey);
});

function tabClick(event)
{
    selectTab(
        $(this).parents('[data-container-sniperfast]').attr('data-container-sniperfast'),
        $(this).attr('data-tab-sniperfast')
    );
}

function selectTab(tabContainer, idTab)
{
    $('[data-container-sniperfast="' + tabContainer + '"] [data-content-sniperfast]').hide();
    $('[data-container-sniperfast="' + tabContainer + '"] [data-tab-sniperfast]').removeClass('active');
    $('[data-container-sniperfast="' + tabContainer + '"] [data-tab-sniperfast="' + idTab + '"]').addClass('active');
    $('[data-container-sniperfast="' + tabContainer + '"] [data-content-sniperfast="' + idTab + '"]').show();
}

function selectDeselectAllLangs()
{
    if ($(this).prop('checked')) {
        $('.snpf-onboard-sel').each(function(){
            $(this).prop('checked', true);
        });
    } else {
        $('.snpf-onboard-sel').each(function(){
            $(this).prop('checked', false);
        });
    }
}

function showGoToModuleButton()
{
    $('#go-to-module-footer').removeClass('sniper_hidden');
}

function activateDeactivateSearch()
{
    let selectedLangs = [];

    $('[id^="snpf-onboard-exportrow-"]:not(".sniper_hidden")').each(function(){
        let langSelId = $(this).attr('id');
        langSelId = langSelId.replace('snpf-onboard-exportrow-', '');
        selectedLangs.push(langSelId);
    });

    var enableSearch = 0;
    if ($('#sniperfast_enable_on').prop('checked')) {
        enableSearch = 1;
    }

    $.ajax({
        url: sniperfast.link.admin_settings_save,
        method: 'POST',
        data: {
            s: sniper_shop_id,
            l: sniper_shop_lang,
            t: sniper_token,
            ajax: true,
            action: 'enableSearch',
            selected_langs: selectedLangs,
            enable_search: enableSearch
        }
    }).done(function(data) {
        if (data.success) {
            if (enableSearch) {
                $('#snpf-try-now-wrapper').removeClass('sniper_hidden');
            } else {
                $('#snpf-try-now-wrapper').addClass('sniper_hidden');
            }
            sniper_alert_data(sniperfast.translate.generic_success, 'n');
        } else {
            sniper_alert_data(sniperfast.translate.error_generic, 'e');
        }
    }).fail(function(jqXHR, textStatus){
        sniper_alert_data(sniperfast.translate.error_generic, 'e');
    });
}


function showWarningGoToModule()
{
    if (!confirm(sniperfast.translate.confirm_go_to_module)) {
        return false;
    }

    window.location.href = $(this).data('src');
}

function registerRecoverAccount()
{
    isOnboarding = true;
    let migration_token = '';
    let data = {};

    if ($('input[name="email"]').length) {
        let email = $('input[name="email"]').val();
        let password = $('input[name="password"]').val();
        if (email == '' || password == '') {
            sniper_alert_data(sniper_account_error, 'e');
            return false;
        }

        data['email'] = email;
        data['password'] = password;
    } else {
        migration_token = $('#sniper_migration_token').text();
        data['migration_token'] = migration_token

        if (migration_token == '') {
            sniper_alert_data(sniper_account_error, 'e');
            return false;
        }
    }

    var action = 'recoverAccount';
    if (requires_conversion) {
        action = 'recoverAccountActive';
        conversionType = 2;
    }

    data =
    {
        ...data,
        ...{
            s: sniper_shop_id,
            l: sniper_shop_lang,
            t: sniper_token,
            ajax: true,
            action: action,
            conversion_type: conversionType,
        }
    }

    $.ajax({
        url: sniperfast.link.admin_settings_save,
        method: 'POST',
        data: data
    }).done(function(data) {
        if (data.success) {
            sniper_alert_data(sniper_account_ok, 'n');
            if (action == 'recoverAccountActive') {
                $('#sniper_first_export_done').slideDown(500).removeClass('sniper_hidden');
                $('#download-first-file').hide();
                $('.sniper_first_export_active_success').removeClass('sniper_hidden');
                $('#sniper_first_export_done .list-info').removeClass('sniper_hidden');

                if ($('#snpf-open-manager').length
                    && !$('#snpf-open-manager').hasClass('snpf_redirect_ok')
                ) {
                    let redirectLinkOpenMgr = $('#snpf-open-manager').attr('href');
                    redirectLinkOpenMgr += '?i=' + data.result.name + '&j=' + data.result.id_job;
                    $('#snpf-open-manager').attr('href', redirectLinkOpenMgr);
                    $('#snpf-open-manager').addClass('snpf_redirect_ok');
                    $('#go-to-manager').removeClass('sniper_hidden');
                }
                return false;
            }
            $.each(data.result, function(idLang, conf){
                $('#snpf-index-'+idLang).val(conf['sniper_index_name']);
                $('#snpf-key-'+idLang).val(conf['sniper_key']);
                $('#snpf-token-'+idLang).val(conf['ajax_token']);
            });
            $('#sniper_first_export').slideDown(500).removeClass('sniper_hidden');
        } else {
            sniper_alert_data(sniper_account_error, 'e');
        }
    }).fail(function(jqXHR, textStatus){
        sniper_alert_data(sniper_account_error, 'e');
    });
}

function registerRecoverAccountStartExp()
{
    $('#sniper_first_export_prods').slideDown(500).removeClass('sniper_hidden');
    $('#sniper_prods_indexing').slideDown(500).removeClass('sniper_hidden');
    $('#snpf-onboard-exportrow-' + sniper_shop_lang).removeClass('sniper_hidden');
    $('#snpf-onboard-elabrow-' + sniper_shop_lang).removeClass('sniper_hidden');
    selectedLangsOnboard.push(sniper_shop_lang);
    ajaxExportLight('&restart');
}

function registerRecoverAccountFull(e)
{
    isOnboarding = true;

    let email = $('input[name="email"]').val();
    let password = $('input[name="password"]').val();
    if (email == '' || password == '') {
        sniper_alert_data(sniper_account_error, 'e');
        return false;
    }

    // conversionType = 1 as default

    $.ajax({
        url: sniperfast.link.admin_settings_save,
        method: 'POST',
        data: {
            s: sniper_shop_id,
            l: sniper_shop_lang,
            t: sniper_token,
            ajax: true,
            email: email,
            password: password,
            action: 'recoverAccountFull'
        }
    }).done(function(data) {
        if (data.success) {
            sniper_alert_data(sniper_account_ok, 'n');
            $.each(data.result, function(idLang, conf){
                $('#snpf-index-'+idLang).val(conf['sniper_index_name']);
                $('#snpf-key-'+idLang).val(conf['sniper_key']);
                $('#snpf-token-'+idLang).val(conf['ajax_token']);
            });
            $(e.target).attr('disabled', 'true');
            $('#sniper_first_export').slideDown(500).removeClass('sniper_hidden');
        } else {
            sniper_alert_data(sniper_account_error, 'e');
        }
    }).fail(function(jqXHR, textStatus){
        sniper_alert_data(sniper_account_error, 'e');
    });
}

function showConfigChangedMessage()
{
    $("#sniper_config_changed").removeClass('sniper_hidden');
}

function hideConfigChangedMessage()
{
    $("#sniper_config_changed").addClass('sniper_hidden');
}

function dedicatedPageEnableChange()
{
    var textValues = ['dedi_page_title', 'dedi_page_rewrite'];
    if ($("#dedi_page_enable_on").is(':checked')) {
        $.each(textValues, function(i, item){
            $("input[type='text'][name='"+ item +"']").parents('.form-group').show();
        });
        $("input[type='radio'][name='lens_redirect_enable']").parents('.form-group').show();
    } else {
        $.each(textValues, function(i, item){
            $("input[type='text'][name='"+ item +"']").parents('.form-group').hide();
        });
        $("input[type='radio'][name='lens_redirect_enable']").parents('.form-group').hide();
    }
}

function sniperStr2url(str, encoding, ucfirst)
{
	str = str.toUpperCase();
	str = str.toLowerCase();
    /* Lowercase */
    str = str.replace(/[\u00E0\u00E1\u00E2\u00E3\u00E4\u00E5\u0101\u0103\u0105\u0430]/g, 'a');
    str = str.replace(/[\u0431]/g, 'b');
    str = str.replace(/[\u00E7\u0107\u0109\u010D\u0446]/g, 'c');
    str = str.replace(/[\u010F\u0111\u0434]/g, 'd');
    str = str.replace(/[\u00E8\u00E9\u00EA\u00EB\u0113\u0115\u0117\u0119\u011B\u0435\u044D]/g, 'e');
    str = str.replace(/[\u0444]/g, 'f');
    str = str.replace(/[\u011F\u0121\u0123\u0433\u0491]/g, 'g');
    str = str.replace(/[\u0125\u0127]/g, 'h');
    str = str.replace(/[\u00EC\u00ED\u00EE\u00EF\u0129\u012B\u012D\u012F\u0131\u0438\u0456]/g, 'i');
    str = str.replace(/[\u0135\u0439]/g, 'j');
    str = str.replace(/[\u0137\u0138\u043A]/g, 'k');
    str = str.replace(/[\u013A\u013C\u013E\u0140\u0142\u043B]/g, 'l');
    str = str.replace(/[\u043C]/g, 'm');
    str = str.replace(/[\u00F1\u0144\u0146\u0148\u0149\u014B\u043D]/g, 'n');
    str = str.replace(/[\u00F2\u00F3\u00F4\u00F5\u00F6\u00F8\u014D\u014F\u0151\u043E]/g, 'o');
    str = str.replace(/[\u043F]/g, 'p');
    str = str.replace(/[\u0155\u0157\u0159\u0440]/g, 'r');
    str = str.replace(/[\u015B\u015D\u015F\u0161\u0441]/g, 's');
    str = str.replace(/[\u00DF]/g, 'ss');
    str = str.replace(/[\u0163\u0165\u0167\u0442]/g, 't');
    str = str.replace(/[\u00F9\u00FA\u00FB\u00FC\u0169\u016B\u016D\u016F\u0171\u0173\u0443]/g, 'u');
    str = str.replace(/[\u0432]/g, 'v');
    str = str.replace(/[\u0175]/g, 'w');
    str = str.replace(/[\u00FF\u0177\u00FD\u044B]/g, 'y');
    str = str.replace(/[\u017A\u017C\u017E\u0437]/g, 'z');
    str = str.replace(/[\u00E6]/g, 'ae');
    str = str.replace(/[\u0447]/g, 'ch');
    str = str.replace(/[\u0445]/g, 'kh');
    str = str.replace(/[\u0153]/g, 'oe');
    str = str.replace(/[\u0448]/g, 'sh');
    str = str.replace(/[\u0449]/g, 'ssh');
    str = str.replace(/[\u044F]/g, 'ya');
    str = str.replace(/[\u0454]/g, 'ye');
    str = str.replace(/[\u0457]/g, 'yi');
    str = str.replace(/[\u0451]/g, 'yo');
    str = str.replace(/[\u044E]/g, 'yu');
    str = str.replace(/[\u0436]/g, 'zh');

    /* Uppercase */
    str = str.replace(/[\u0100\u0102\u0104\u00C0\u00C1\u00C2\u00C3\u00C4\u00C5\u0410]/g, 'A');
    str = str.replace(/[\u0411]/g, 'B');
    str = str.replace(/[\u00C7\u0106\u0108\u010A\u010C\u0426]/g, 'C');
    str = str.replace(/[\u010E\u0110\u0414]/g, 'D');
    str = str.replace(/[\u00C8\u00C9\u00CA\u00CB\u0112\u0114\u0116\u0118\u011A\u0415\u042D]/g, 'E');
    str = str.replace(/[\u0424]/g, 'F');
    str = str.replace(/[\u011C\u011E\u0120\u0122\u0413\u0490]/g, 'G');
    str = str.replace(/[\u0124\u0126]/g, 'H');
    str = str.replace(/[\u0128\u012A\u012C\u012E\u0130\u0418\u0406]/g, 'I');
    str = str.replace(/[\u0134\u0419]/g, 'J');
    str = str.replace(/[\u0136\u041A]/g, 'K');
    str = str.replace(/[\u0139\u013B\u013D\u0139\u0141\u041B]/g, 'L');
    str = str.replace(/[\u041C]/g, 'M');
    str = str.replace(/[\u00D1\u0143\u0145\u0147\u014A\u041D]/g, 'N');
    str = str.replace(/[\u00D3\u014C\u014E\u0150\u041E]/g, 'O');
    str = str.replace(/[\u041F]/g, 'P');
    str = str.replace(/[\u0154\u0156\u0158\u0420]/g, 'R');
    str = str.replace(/[\u015A\u015C\u015E\u0160\u0421]/g, 'S');
    str = str.replace(/[\u0162\u0164\u0166\u0422]/g, 'T');
    str = str.replace(/[\u00D9\u00DA\u00DB\u00DC\u0168\u016A\u016C\u016E\u0170\u0172\u0423]/g, 'U');
    str = str.replace(/[\u0412]/g, 'V');
    str = str.replace(/[\u0174]/g, 'W');
    str = str.replace(/[\u0176\u042B]/g, 'Y');
    str = str.replace(/[\u0179\u017B\u017D\u0417]/g, 'Z');
    str = str.replace(/[\u00C6]/g, 'AE');
    str = str.replace(/[\u0427]/g, 'CH');
    str = str.replace(/[\u0425]/g, 'KH');
    str = str.replace(/[\u0152]/g, 'OE');
    str = str.replace(/[\u0428]/g, 'SH');
    str = str.replace(/[\u0429]/g, 'SHH');
    str = str.replace(/[\u042F]/g, 'YA');
    str = str.replace(/[\u0404]/g, 'YE');
    str = str.replace(/[\u0407]/g, 'YI');
    str = str.replace(/[\u0401]/g, 'YO');
    str = str.replace(/[\u042E]/g, 'YU');
    str = str.replace(/[\u0416]/g, 'ZH');

    str = str.toLowerCase();

    str = str.replace(/[^a-z0-9\s\'\:\/\[\]-]/g,'');
	str = str.replace(/[\u0028\u0029\u0021\u003F\u002E\u0026\u005E\u007E\u002B\u002A\u002F\u003A\u003B\u003C\u003D\u003E]/g, '');
	str = str.replace(/[\s\'\:\/\[\]-]+/g, ' ');

	// Add special char not used for url rewrite
	str = str.replace(/[ ]/g, '-');
	str = str.replace(/[\/\\"'|,;%]*/g, '');

	if (ucfirst == 1) {
		var first_char = str.charAt(0);
		str = first_char.toUpperCase()+str.slice(1);
	}

	return str;
}

function showHideLandingSourceData()
{
    var sourceVal = $('#landing_source').val();
    if (sourceVal == 0 || sourceVal == 1) {
        $('#landing_source_data').closest('.form-group').hide();
    } else {
        $('#landing_source_data').closest('.form-group').show();
    }
}

function fetchIndexingStatus()
{
    if (typeof last_job_id === 'undefined'
        || last_job_id == ''
        || !$('#export').length
    ) {
        return false;
    }
    $.ajax({
        url: sniper_search_url+'?id_job='+last_job_id,
        method: 'POST',
        data: {
            shop:   sniper_shop_id,
            lang:   $("#lang-selector").val(),
            key:    sniper_key,
            index:  sniper_index_name,
            token:  sniper_token,
            ip:     sniper_source_ip
        }
    }).done(function (data) {
        switch (data.status) {
            case 'pending':
                $("#gen-index-status").text(sniper_index_pending);
                $("#gen-add-on").text(data.date_add);
                break;
            case 'not found':
                $("#gen-index-status").text(sniper_index_notfound);
                break;
            case 'completed':
                $("#gen-index-status").text(sniper_index_completed);
                $("#gen-elab-prod").text(data.number_of_entities);
                $("#gen-elab-time").text(data.time_elapsed);
                $("#gen-comp-on").text(data.date_completed);
                saveFeaturesRange(data.facets);
                break;
            default:
                break;
        }
    })
}

function validateIP(ipAddress){
    var ipv4Re = new RegExp('^((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(\.|$)){4}', 'i');
    if (ipv4Re.test(ipAddress)) {
        return true;
    } else {
        return false;
    }
}

function saveFeaturesRange(rangeData)
{
    $.ajax({
        url: sniperfast.link.admin_settings_save,
        method: 'POST',
        data: {
            s: sniper_shop_id,
            l: sniper_shop_lang,
            t: sniper_token,
            ajax: true,
            action: 'featureRange',
            val: rangeData,
        }
    }).done(function(data) {
        if (data.success) {
            console.log(sniper_data_saved);
        } else {
            console.log(sniper_data_save_error);
        }
    }).fail(function(){
        console.log(sniper_data_save_error);
    });
}

function showAdvancedCron()
{
    $('.sniper-adavanced-cron').show();
}

function exportBtnClick()
{
    if (confirm(sniper_exportAlert)) {
        resetStats();
        $('#export-info').addClass('expanded');

        $(this).addClass('disabled');
        $('#use_cron_mode').prop('disabled', true);

        setStatusMessage('working', 'working');

        ajaxExport('&restart');
    }
}

function ajaxExport(option)
{
    var url = sniper_ajaxExportUrl + option;

    if ($('#use_cron_mode').prop('checked')) {
        url += '&cron&restart';
    }

    $.ajax({
        url: url
    }).done(function(data) {
        processAjaxExportReturn(data, true);
    }).fail(function() {
        processAjaxExportReturn(null, false);
    });
}

function processAjaxExportReturn(data, success)
{
    sniper_exportInfo['cycles']++;

    if (success) {
        if (data['success'] == true) {
            if (data['msg'] == 'export_done') {
                setStatusMessage('done', 'done');
                $('#btn-export').removeClass('disabled');
                $('#use_cron_mode').prop('disabled', false);
                sendFetchRequest(0);
                hideConfigChangedMessage();
            } else if (data['msg'] == 'export_cycle_done') {
                setStatusMessage('working', 'working');
                ajaxExport('');
            } else {
                setStatusMessage('done', data['msg']);
            }

            sniper_exportInfo['time_total']        += data['time_total'];
            sniper_exportInfo['exported_csv_rows'] += data['exported_csv_rows'];
            sniper_exportInfo['skipped_price']     += data['skipped_price'];
            sniper_exportInfo['skipped_quantity']  += data['skipped_quantity'];
            sniper_exportInfo['skipped_inactive']  += data['skipped_inactive'];

            $('#export-cycles')          .html(sniper_exportInfo['cycles']);
            $('#export-total-time')      .html(sniper_exportInfo['time_total'].toFixed(3));
            $('#export-avg-time')        .html(data['time_product_avg']);
            $('#export-memory-use')      .html(data['max_used_memory']);
            $('#export-csv-rows')        .html(sniper_exportInfo['exported_csv_rows']);
            $('#export-last-id')         .html(data['last_id_product']);
            $('#export-products-count')  .html(data['products_to_export']);
            $('#export-skipped-price')   .html(sniper_exportInfo['skipped_price']);
            $('#export-skipped-quantity').html(sniper_exportInfo['skipped_quantity']);
            $('#export-skipped-inactive').html(sniper_exportInfo['skipped_inactive']);

            var processedProducts = data['skipped_already_exported']
                + data['skipped_price']
                + data['skipped_quantity']
                + data['skipped_inactive']
                + data['exported_csv_rows'];

            var progress = 100*processedProducts/data['products_to_export'];
            setProgressbarValue(progress, false, 0);
        } else if (data['success'] == false) {
            if (data['msg'] == 'another_instance_running') {
                setStatusMessage('error', data['info']);
            } else {
                setStatusMessage('error', data['msg']);
            }
        }
    } else {
        setStatusMessage('error', 'cannot_execute_request');
    }
}
function startMultipleExports()
{
    selectedLangsOnboard = [];
    $('.snpf-onboard-sel').each(function(){
        if ($(this).prop('checked')) {
            let checkId = $(this).attr('id');
            checkId = checkId.replace('snpf-onboard-sel-', '');
            selectedLangsOnboard.push(checkId);
            $('#snpf-onboard-exportrow-' + checkId).removeClass('sniper_hidden');
            $('#snpf-onboard-elabrow-' + checkId).removeClass('sniper_hidden');
        }
    });

    if (selectedLangsOnboard.length == 0) {
        sniper_alert_data(sniperNoLangSelected, 'w');
        return false;
    }

    $('#sniper_first_export_prods').slideDown(500).removeClass('sniper_hidden');
    $('#sniper_prods_indexing').slideDown(500).removeClass('sniper_hidden');
    ajaxExportLight('&restart');
}

function ajaxExportLight(option)
{
    var url = sniperfast.link.ajax_export_link + option + '&forced_lang=' + selectedLangsOnboard[0];

    $.ajax({
        url: url
    }).done(function(data) {
        processAjaxExportReturnLight(data, true);
    }).fail(function() {
        processAjaxExportReturnLight(null, false);
    });
}

function processAjaxExportReturnLight(data, success)
{
    if (success) {
        if (data['success'] == true) {
            if (data['msg'] == 'export_done') {
                sendFetchRequest(selectedLangsOnboard[0]);
                $('#snpf-onboard-exportrow-' + selectedLangsOnboard[0] + ' .sniperfast-icon-enabled').removeClass('sniper_hidden');
                $('#snpf-onboard-exportrow-' + selectedLangsOnboard[0] + ' .sniperfast-icon-disabled').addClass('sniper_hidden');
                setProgressbarValue(100, true, selectedLangsOnboard[0]);

                selectedLangsOnboard = selectedLangsOnboard.slice(1);
                if (selectedLangsOnboard.length > 0) {
                    ajaxExportLight('&restart');
                } else {
                    // showExportDonePanel(true);
                }
            } else if (data['msg'] == 'export_cycle_done') {
                var processedProducts = data['skipped_already_exported']
                    + data['skipped_price']
                    + data['skipped_quantity']
                    + data['skipped_inactive']
                    + data['exported_csv_rows'];

                var progress = 100*processedProducts/data['products_to_export'];
                setProgressbarValue(progress, true, selectedLangsOnboard[0]);

                ajaxExportLight('');
            } else {
                sniper_alert_data(data['msg'], 'n');
                // showExportDonePanel(true);
            }
        } else if (data['success'] == false) {
            $('#snpf-onboard-exportrow-' + selectedLangsOnboard[0] + ' .sniperfast-icon-disabled').removeClass('sniper_hidden');
            $('#snpf-onboard-exportrow-' + selectedLangsOnboard[0] + ' .sniperfast-icon-enabled').addClass('sniper_hidden');
            if (data['msg'] == 'another_instance_running') {
                sniper_alert_data(data['info'], 'e');
            } else {
                sniper_alert_data(data['msg'], 'e');
            }
            // showExportDonePanel(false);
        }
    } else {
        sniper_alert_data('cannot_execute_request', 'e');
    }
}

// function showIndexStatusPanel(shop_id = 0)
// {
//     if($('#sniper_prods_indexing').hasClass('sniper_hidden')) {
//         $('#sniper_prods_indexing').slideDown(500).removeClass('sniper_hidden')
//         getLastJobStatus(shop_id);
//     }
// }

function showExportDonePanel(success)
{
    $('#sniper_first_export_done').slideDown(500).removeClass('sniper_hidden');
    if (success) {
        $('.sniper_first_export_success').removeClass('sniper_hidden');
        $('#sniper_first_export_done .list-info').removeClass('sniper_hidden');
    } else {
        $('.sniper_first_export_error').removeClass('sniper_hidden');
    }
}

function resetStats()
{
    sniper_exportInfo = {
        "time_total": 0,
        "exported_csv_rows": 0,
        "skipped_price": 0,
        "skipped_quantity": 0,
        "skipped_inactive": 0,
        "cycles": 0
    };

    $('#export-cycles')          .html('--');
    $('#export-total-time')      .html('--');
    $('#export-avg-time')        .html('--');
    $('#export-memory-use')      .html('--');
    $('#export-csv-rows')        .html('--');
    $('#export-last-id')         .html('--');
    $('#export-products-count')  .html('--');
    $('#export-skipped-price')   .html('--');
    $('#export-skipped-quantity').html('--');
    $('#export-skipped-inactive').html('--');

    setProgressbarValue(0, false, 0);
}

function setProgressbarValue(value, isLight, idLang)
{
    var bar = null;
    var barWidth = null;
    if (!isLight) {
        bar = $('#export-progress-bar > .progress-bar-inner');
        barWidth = $('#export-progress-bar').width();
    } else {
        bar = $('#export-progress-bar-light-' + idLang + ' > .progress-bar-inner');
        barWidth = $('#export-progress-bar-light-' + idLang).width();
    }

    var newWidth = Math.round(value*barWidth/100);
    bar.width(newWidth+'px');
}

function setStatusMessage(type, message)
{
    var label = $('#export-status');
    var color = '';

    switch (type) {
        case 'done':
            color = 'green';
            break;
        case 'working':
            color = 'prestalia-green';
            break;
        case 'error':
            color = 'red';
            break;
    }

    if (message in sniper_statusText) {
        message = sniper_statusText[message];
    }

    label.html('<span class="'+color+'">'+message+'</span>');
}

function sendFetchRequest(forcedId)
{
    let fetchLang = '';
    let indexName = '';
    let indexKey = '';
    let ajaxToken = '';
    if (forcedId == 0 || forcedId == '0') {
        fetchLang = $("#lang-selector").val();
        indexName = sniper_index_name;
        indexKey = sniper_key;
        ajaxToken = sniper_token;
    } else {
        fetchLang = forcedId;
        indexName = $('#snpf-index-'+forcedId).val();
        indexKey = $('#snpf-key-'+forcedId).val();
        ajaxToken = $('#snpf-token-'+forcedId).val();
    }

    let data = {};
    let requestUrl = '';
    var isInDevelopment = $('#is_in_development_on').prop('checked');

    if (isInDevelopment) {
        requestUrl = sniperfast.link.admin_settings_save;
        data = {
            action: 'indexRequestToManager',
            s: sniper_shop_id,
            l: fetchLang,
            t: ajaxToken,
            ajax: true,
            ip: sniper_source_ip,
            v: sniper_module_version
        }
    } else {
        requestUrl = sniper_search_url;
        data = {
            shop: sniper_shop_id,
            lang: fetchLang,
            key: indexKey,
            index: indexName,
            token: ajaxToken,
            ip: sniper_source_ip,
            v: sniper_module_version
        }
    }

    $.ajax({
        url: requestUrl,
        method: 'POST',
        data: data
    }).done(function (data) {
        if (!isInDevelopment && data.status == 'ok') {
            sniper_alert_data(sniper_data_success, 'n');
            saveJobId(data.id_job, forcedId).then(function () {
                getLastJobStatus(forcedId);

                if ($('#snpf-open-manager').length
                    && !$('#snpf-open-manager').hasClass('snpf_redirect_ok')
                ) {
                    let redirectLinkOpenMgr = $('#snpf-open-manager').attr('href');
                    redirectLinkOpenMgr += '?i=' + $('#snpf-index-'+forcedId).val() + '&j=' + data.id_job;
                    $('#snpf-open-manager').attr('href', redirectLinkOpenMgr);
                    $('#snpf-open-manager').addClass('snpf_redirect_ok');
                }
            });
        } else if (isInDevelopment && data.success) {
            sniper_alert_data(sniper_data_success, 'n');
            saveJobId(data.result.id_job, forcedId).then(function () {
                getLastJobStatus(forcedId);

                if ($('#snpf-open-manager').length
                    && !$('#snpf-open-manager').hasClass('snpf_redirect_ok')
                ) {
                    let redirectLinkOpenMgr = $('#snpf-open-manager').attr('href');
                    redirectLinkOpenMgr += '?i=' + $('#snpf-index-'+forcedId).val() + '&j=' + data.result.id_job;
                    $('#snpf-open-manager').attr('href', redirectLinkOpenMgr);
                    $('#snpf-open-manager').addClass('snpf_redirect_ok');
                }
            });
        } else {
            $('#elab-progress-status-'+forcedId).attr('status', 'finished');
            $('#snpf-onboard-elabrow-'+forcedId+' .sniperfast-icon-disabled').removeClass('sniper_hidden');
            if ($('#snpf-open-manager').length
                && !$('#snpf-open-manager').hasClass('snpf_redirect_ok')
            ) {
                let redirectLinkOpenMgr = $('#snpf-open-manager').attr('href');
                redirectLinkOpenMgr += '?i=' + $('#snpf-index-'+forcedId).val() + '&j=0';
                $('#snpf-open-manager').attr('href', redirectLinkOpenMgr);
                $('#snpf-open-manager').addClass('snpf_redirect_ok');
            }
            sniper_alert_data(sniper_data_error, 'e');
        }
    }).fail(function () {
        sniper_alert_data(sniper_data_error, 'e');
    });
}

function getLastJobStatus(forcedId = 0)
{
    if (forcedId == 0) {
        return;
    }
    var hideClass = 'sniper_hidden';
    var indexLang = forcedId;
    var indexName = $('#snpf-index-'+forcedId).val();
    var indexKey = $('#snpf-key-'+forcedId).val();
    var idJob = $('#snpf-job-'+forcedId).val();

    $('#snpf-onboard-elabrow-'+forcedId+' .last_job_status').each(function(){
        $(this).addClass(hideClass);
    });

    $.ajax({
        url: sniper_search_url + '?id_job=' + idJob,
        method: 'POST',
        data: {
            shop: sniper_shop_id,
            lang: indexLang,
            key: indexKey,
            index: indexName,
            token: '-',
            ip: '127.0.0.1'
        }
    }).done(function(data) {
        switch (data.status) {
            case 'pending':
                if (typeof data.step !== 'undefined') {
                    data.status += '_' + data.step;
                    if (data.step == 0) {
                        $('#elab-progress-status-'+forcedId+' .last_job_status_pending').removeClass(hideClass);
                    } else if (data.step == 1) {
                        $('#elab-progress-status-'+forcedId+' .last_job_status_working').removeClass(hideClass);
                    }
                } else {
                    $('#elab-progress-status-'+forcedId+' .last_job_status_pending').removeClass(hideClass);
                }
                break;
            case 'not found':
                $('#elab-progress-status-'+forcedId+' .last_job_status_notfound').removeClass(hideClass);
                $('#snpf-job-'+indexLang).attr('status', 'notfound');
                $('#elab-progress-status-'+forcedId).attr('status', 'finished');
                $('#snpf-onboard-elabrow-'+forcedId+' .sniperfast-icon-disabled').removeClass(hideClass);
                break;
            case 'completed':
                $('#snpf-job-'+indexLang).attr('status', 'completed');
                $('#elab-progress-status-'+forcedId).attr('status', 'finished');
                $('#snpf-onboard-elabrow-'+forcedId+' .sniperfast-icon-enabled').removeClass(hideClass);
                break;
            default:
                break;
        }
        setIndexProgressbar(data.status, forcedId);
        let currentStatus = data.status;

        // If current is pending or not all items have finished
        if (~currentStatus.indexOf('pending')) {
            setTimeout(function(forcedId) { getLastJobStatus(forcedId); }, 5000, forcedId);
        } else {
            let elaborationFinishedCount = 0;
            let completedCount = 0;
            let selectedLangs = [];

            $('[id^="snpf-onboard-exportrow-"]:not(".sniper_hidden")').each(function(){
                let langSelId = $(this).attr('id');
                langSelId = langSelId.replace('snpf-onboard-exportrow-', '');
                selectedLangs.push(langSelId);
            });

            $('[id^="snpf-onboard-elabrow-"]:not(".sniper_hidden")').each(function(){
                if ($(this).find('.elab-progress-status').attr('status') == 'finished') {
                    ++elaborationFinishedCount;
                }
            });

            if (selectedLangs.length == elaborationFinishedCount) {
                $.each(selectedLangs, function(i, item){
                    if ($('#snpf-job-'+item).attr('status') == 'completed') {
                        ++completedCount;
                    }
                });

                if (completedCount == elaborationFinishedCount) {
                    showExportDonePanel(true);
                } else {
                    showExportDonePanel(false);
                }
            }
        }
    });
}

function setIndexProgressbar(status, forcedId) {
    let bar = $('#elab-progress-bar-light-'+forcedId+' > .progress-bar-inner');
    barWidth = $('#elab-progress-bar-light-' + forcedId).width();

    var value = 0;

    switch (status) {
        case 'pending':
            bar.css('width', '50%');
            value = 50;
            break;
        case 'pending_0':
            bar.css('width', '30%');
            $('#indexing_status_percentage').text('30');
            value = 30;
            break;
        case 'pending_1':
            bar.css('width', '60%');
            $('#indexing_status_percentage').text('60');
            value = 60;
            break;
        case 'not found':
            bar.css('width', '0%');
            value = 0;
            break;
        case 'completed':
            bar.css('width', '100%');
            value = 100;
            break;
    }

    var newWidth = Math.round(value*barWidth/100);
    bar.width(newWidth+'px');
}


function saveJobId(idJob, forcedId)
{
    let fetchLang = '';
    let ajaxToken = '';
    if (forcedId == 0 || forcedId == '0') {
        fetchLang = sniper_shop_lang;
        ajaxToken = sniper_token;
    } else {
        fetchLang = forcedId;
        ajaxToken = $('#snpf-token-'+forcedId).val();
    }

    let selectedLangs = [];
    $('.snpf-onboard-sel').each(function(){
        if ($(this).prop('checked')) {
            let langSelId = $(this).attr('id');
            langSelId = langSelId.replace('snpf-onboard-sel-', '');
            selectedLangs.push(langSelId);
        }
    });

    return $.ajax({
        url: sniperfast.link.admin_settings_save,
        method: 'POST',
        data: {
            s: sniper_shop_id,
            l: fetchLang,
            t: ajaxToken,
            ajax: true,
            action: 'saveJob',
            id_job: idJob,
            is_onboarding: isOnboarding,
            status_sent: statusRequestSent,
            conversion_type: conversionType,
            selected_langs: selectedLangs
        }
    }).done(function(data) {
        if (data.success) {
            $(`#snpf-job-${forcedId}`).val(idJob);
            statusRequestSent = true;
            sniper_alert_data(sniper_idjob_success, 'n');
        } else {
            sniper_alert_data(sniper_idjob_error, 'e');
        }
    }).fail(function(){
        sniper_alert_data(sniper_idjob_error, 'e');
    });
}

// function convertSniperIndex() {
//     if (!confirm(sniper_confirm_conversion)) {
//         return false;
//     }

//     $.ajax({
//         url: sniperfast.link.admin_settings_save,
//         method: 'POST',
//         data: {
//             s: sniper_shop_id,
//             l: sniper_shop_lang,
//             t: sniper_token,
//             ajax: true,
//             action: 'convertIndex'
//         }
//     }).done(function(data) {
//         if (data.success) {
//             sniper_alert_data(sniperfast.translate.conversion_success, 'n');
//             location.reload();
//         } else {
//             sniper_alert_data(sniperfast.translate.error_generic, 'e');
//         }
//     }).fail(function(jqXHR, textStatus){
//         sniper_alert_data(sniperfast.translate.error_generic, 'e');
//     });
// }

function deleteIndexNameAndKey()
{
    if (!confirm(sniper_confirm_action)) {
        return false;
    }

    $.ajax({
        url: sniperfast.link.admin_settings_save,
        method: 'POST',
        data: {
            s: sniper_shop_id,
            l: sniper_shop_lang,
            t: sniper_token,
            ajax: true,
            action: 'deleteIndexAndKey'
        }
    }).done(function(data) {
        if (data.success) {
            sniper_alert_data(sniperfast.translate.generic_success, 'n');
            location.reload();
        } else {
            sniper_alert_data(sniperfast.translate.error_generic, 'e');
        }
    }).fail(function(jqXHR, textStatus){
        sniper_alert_data(sniperfast.translate.error_generic, 'e');
    });
}

function detectParentDivAttribute(event)
{
    var attribute = $(event.target).closest('div').attr('id');
    if (attribute === '' || typeof attribute === 'undefined') {
        attribute = $(event.target).closest('div').attr('class');
    }
    $("#search-bar-lookup-result").text(attribute);
}

function languageSelectorChange()
{
    $.ajax({
        url: sniperfast.link.admin_settings_save,
        method: 'POST',
        data: {
            s: sniper_shop_id,
            l: sniper_shop_lang,
            t: sniper_token,
            ajax: true,
            action: 'selectorChange',
            lang: $('#lang-selector').val()
        }
    }).done(function(data) {
        if (data.success) {
            sniper_alert_data(sniper_data_saved, 'n');
            location.reload();
        } else {
            sniper_alert_data(sniperfast.translate.error_generic, 'e');
            $('#lang-selector').prop('disabled', false);
        }
    }).fail(function(jqXHR, textStatus){
        sniper_alert_data(sniperfast.translate.error_generic, 'e');
        $('#lang-selector').prop('disabled', false);
    });
}

function sniper_alert_data(message, type)
{
    if (typeof($.growl) != 'undefined') {
        switch (type) {
            case 'e':
                $.growl.error({title: '', message: message});
                break;
            case 'n':
                $.growl.notice({title: '', message: message});
                break;
            case 'w':
                $.growl.warning({title: '', message: message});
                break;
        }
    } else {
        alert('Error: '+ message);
    }
}

function ajaxCheckboxLiteClick()
{
    if ($(this).hasClass('sniperfast-icon-loading')) {
        return;
    }
    var attrIdFull = $(this).attr('id');
    let attrId = attrIdFull.replace('sniperfast-chk-', '');
    attrId = attrId.split('-');
    var type = attrId[0];
    if (type == 'filter' || type == 'open') {
        $('#positions .sniperfast-ajax-checkbox-lite').each(function(i, item){
            let currentId = $(this).attr('id');
            if (currentId == attrIdFull) {
                return true;
            }

            $(this).removeClass($(this).data('prevstat'));
            $(this).addClass('sniperfast-icon-loading');
        });
    } else {
        $(this).removeClass($(this).data('prevstat'));
        $(this).addClass('sniperfast-icon-loading');
    }

    $.ajax({
        url: sniperfast.link.admin_settings_save,
        method: 'POST',
        data: {
            s: sniper_shop_id,
            l: sniper_shop_lang,
            t: sniper_token,
            ajax: true,
            type: type,
            id: attrId[1],
            action: 'flipStatus'
        }
    }).done(function(data){
        if (type == 'filter' || type == 'open') {
            if (data.success) {
                if ($('#' + attrIdFull).hasClass('sniperfast-icon-disabled')) {
                    $('#' + attrIdFull).removeClass('sniperfast-icon-disabled');
                    $('#' + attrIdFull).addClass('sniperfast-icon-enabled');
                    $('#' + attrIdFull).data('prevstat', 'sniperfast-icon-enabled');
                } else {
                    $('#' + attrIdFull).removeClass('sniperfast-icon-enabled');
                    $('#' + attrIdFull).addClass('sniperfast-icon-disabled');
                    $('#' + attrIdFull).data('prevstat', 'sniperfast-icon-disabled');
                }
            } else {
                sniper_alert_data(sniperfast.translate.error_generic, 'e');
            }

            $('#positions .sniperfast-icon-loading').each(function(i, item){
                $(this).removeClass('sniperfast-icon-loading');
                $(this).addClass($(this).data('prevstat'));
            });
        } else {
            if (data.success) {
                $('#' + attrIdFull).removeClass('sniperfast-icon-loading');
                if ($('#' + attrIdFull).data('prevstat') == 'sniperfast-icon-disabled') {
                    $('#' + attrIdFull).addClass('sniperfast-icon-enabled');
                    $('#' + attrIdFull).data('prevstat', 'sniperfast-icon-enabled');
                } else {
                    $('#' + attrIdFull).addClass('sniperfast-icon-disabled');
                    $('#' + attrIdFull).data('prevstat', 'sniperfast-icon-disabled');
                }
            } else {
                sniper_alert_data(sniperfast.translate.error_generic, 'e');
            }
        }
    }).fail(function(){
        $('.sniperfast-icon-loading').each(function(i, item){
            $(this).removeClass('sniperfast-icon-loading');
            $(this).addClass($(this).data('prevstat'));
        });
        sniper_alert_data(sniperfast.translate.error_generic, 'e');
    });
}

function checkLatestVersion()
{
    $.ajax({
        url: 'https://app.prestalia.it/tokenhandler/getLatestVersion.php',
        method: 'POST',
        data: {
            id_module: 5,
            iso_code: sniperfast_lang_iso_code
        }
    }).done(function(data) {
        if (!data.success || data.msg == 'not_found') {
            return false;
        }

        if (versionCompare(sniper_module_version, data.msg) == -1) {
            if (data.custom_msg != '') {
                customMsg = '.<br/><span class="sniper-red">!!!</span> '+
                data.custom_msg
                +' <span class="sniper-red">!!!</span>';
            }
            var newVersionWarning =
                '<div style="text-align:center;font-weight:bold;font-size:1.2em;">' +
                    '<span class="sniper-yellow">' + sniper_version_news + '</span>' +
                    ' ' + sniper_version_warning + ' <span class="sniper-green">' + data.msg + '</span>' +
                    customMsg +
                '</div>';
            $('.sniperfast-lang-selector').append(newVersionWarning);
        }
    });
}

function versionCompare(v1, v2) {
    const v1Arr = v1.split('.').map(Number);
    const v2Arr = v2.split('.').map(Number);

    for (let i = 0; i < Math.max(v1Arr.length, v2Arr.length); i++) {
        if (v1Arr[i] > v2Arr[i]) return 1;
        if (v1Arr[i] < v2Arr[i]) return -1;
    }

    return 0;
}

function checkSniperfastExpirationDateFromModule()
{
    $.ajax({
        url: sniperfast.link.admin_settings_save,
        method: 'POST',
        data: {
            s: sniper_shop_id,
            l: sniper_shop_lang,
            t: sniper_token,
            ajax: true,
            action: 'getExpirationDate',
            isFromBo: 1
        }
    }).done(function(data){
        if (typeof data.success === 'undefined' || !data.success) {
            return;
        }

        if (data.result.has_expired) {
            $('#sniperfast-expiration-past').removeClass('sniper_hidden');
            $('#sniperfast-expiration-summary').removeClass('sniper_hidden');
            return;
        }

        if (data.result.show_warning) {
            if (data.result.expiration_days > 1) {
                $('.exp_days').text(data.result.expiration_days);
                $('.exp_days_text').removeClass('sniper_hidden');
            } else if (data.result.expiration_days == 1) {
                $('.exp_tomorrow').removeClass('sniper_hidden');
            } else if (data.result.expiration_days == 0) {
                $('.exp_today').removeClass('sniper_hidden');
            }
            $('#sniperfast-expiration-upcoming').removeClass('sniper_hidden');
            $('#sniperfast-expiration-summary').removeClass('sniper_hidden');
            if (data.result.custom_message != '') {
                $('#sniperfast-expiration-msg').html(data.result.custom_message);
                $('#sniperfast-expiration-msg').removeClass('sniper_hidden');
            }
        }
    }).fail(function(){
        console.log('Could not retrieve SniperFast\'s expiration date');
    });
}

function handleBoWarnChange()
{
    let isChecked = 0;
    if ($('#hide_bowarn_enable_on').prop('checked')) {
        isChecked = 1;
    }
    $.ajax({
        url: sniperfast.link.admin_settings_save,
        method: 'POST',
        data: {
            s: sniper_shop_id,
            l: sniper_shop_lang,
            t: sniper_token,
            ajax: true,
            checked: isChecked,
            action: 'saveBoWarning'
        }
    }).done(function(data){
        if (typeof data.success === 'undefined') {
            sniper_alert_data(sniperfast.translate.error_generic, 'e');
            return;
        }

        if (data.success) {
            sniper_alert_data(sniper_data_saved, 'n');
        } else {
            sniper_alert_data(sniperfast.translate.error_generic, 'e');
        }
    }).fail(function(){
        sniper_alert_data(sniperfast.translate.error_generic, 'e');
    });
}