/**
 *  Copyright (C) Prestalia - All Rights Reserved
 *  Unauthorized copying of this file, via any medium is strictly prohibited
 *  Proprietary and confidential
 *
 *  @author    Prestalia <prestalia.it>
 *  @copyright 2019 - 2024 Prestalia
 *  @license   Commercial
 */
(() => {
    var cdnBaseSniperfast = "https://cdn.sniperfast.com/";

    function appendResource(resource, configuration = {})
    {
        var config = {...{
            "dependencies": "",
            "async": false,
            "in_footer": false,
            "id": null,
        }, ...configuration};

        var type = _detectResourceType(resource);

        var element = _createElement(type, resource, config);
        _appendElement(element, config);
    }

    function _detectResourceType(resources)
    {
        var extension = resources.split('.').pop().trim();

        switch(extension) {
            case 'css':
                return 'stylesheet'
            case 'js':
                return 'text/javascript'
        }
    }

    function _createElement(type, resource, config)
    {
        params = new URLSearchParams({ v: Date.now() });

        if (type === 'stylesheet') {
            var link = document.createElement('link');
            link.rel ='stylesheet';
            link.href = resource + '?' + params.toString();
            if (config.id) {
                link.id = config.id;
            }
            return link;
        } else if (type === 'text/javascript') {
            var script = document.createElement('script');
            script.src = resource + '?' + params.toString();
            script.async = config.async;
            if (config.id) {
                script.id = config.id;
            }
            return script;
        }
    }

    function _appendElement(element, config)
    {
        if (config.in_footer) {
            document.body.appendChild(element);
        } else {
            document.head.appendChild(element);
        }
    }

    function loadResources()
    {
        jsPath = 'v2/200/sniperfast.js';
        if (typeof prestalia_sniperfast.custom_resources.js !== 'undefined') {
            if (prestalia_sniperfast.custom_resources.js.trim() !== '') {
                jsPath = prestalia_sniperfast.custom_resources.js
            }
        }

        cssPath = 'v2/200/sniperfast.css'
        if (typeof prestalia_sniperfast.custom_resources.css !== 'undefined') {
            if (prestalia_sniperfast.custom_resources.css.trim() !== '') {
                cssPath = prestalia_sniperfast.custom_resources.css
            }
        }

        var resources = {
            'js': [ cdnBaseSniperfast + jsPath ],
            'css': [ cdnBaseSniperfast + cssPath ]
        };

        if (!Array.isArray(resources['js'])) {
            resources['js'] = [resources['js']];
        }
        if (!Array.isArray(resources['css'])) {
            resources['css'] = [resources['css']];
        }

        resources['js'].forEach(function (resource) {
            appendResource(resource, {'dependencies' : ['jquery', 'jquery-ui', 'jquery-ui-slider'], 'id': ['sniperfast-script']});
        });

        resources['css'].forEach(function (resource) {
            appendResource(resource, {'id': 'sniperfast-style'});
        });
    }
    loadResources();
})();